/*!
 * Copyright 2015 Savv-e Elearning.
 * http://www.savv-e.com.au/
 *
 * A powerful eLearning framework.
 */


if (!window.localStorage) {
    //@see https://developer.mozilla.org/en/docs/Web/Guide/API/DOM/Storage
    Object.defineProperty(window, "localStorage", new (function () {
        var aKeys = [], oStorage = {};
        Object.defineProperty(oStorage, "getItem", {
            value: function (sKey) { return sKey ? this[sKey] : null; },
            writable: false,
            configurable: false,
            enumerable: false
        });
        Object.defineProperty(oStorage, "key", {
            value: function (nKeyId) { return aKeys[nKeyId]; },
            writable: false,
            configurable: false,
            enumerable: false
        });
        Object.defineProperty(oStorage, "setItem", {
            value: function (sKey, sValue) {
                if(!sKey) { return; }
                document.cookie = escape(sKey) + "=" + escape(sValue) + "; expires=Tue, 19 Jan 2038 03:14:07 GMT; path=/";
            },
            writable: false,
            configurable: false,
            enumerable: false
        });
        Object.defineProperty(oStorage, "length", {
            get: function () { return aKeys.length; },
            configurable: false,
            enumerable: false
        });
        Object.defineProperty(oStorage, "removeItem", {
            value: function (sKey) {
                if(!sKey) { return; }
                document.cookie = escape(sKey) + "=; expires=Thu, 01 Jan 1970 00:00:00 GMT; path=/";
            },
            writable: false,
            configurable: false,
            enumerable: false
        });
        this.get = function () {
            var iThisIndx;
            for (var sKey in oStorage) {
                iThisIndx = aKeys.indexOf(sKey);
                if (iThisIndx === -1) { oStorage.setItem(sKey, oStorage[sKey]); }
                else { aKeys.splice(iThisIndx, 1); }
                delete oStorage[sKey];
            }
            for (aKeys; aKeys.length > 0; aKeys.splice(0, 1)) { oStorage.removeItem(aKeys[0]); }
            for (var aCouple, iKey, nIdx = 0, aCouples = document.cookie.split(/\s*;\s*/); nIdx < aCouples.length; nIdx++) {
                aCouple = aCouples[nIdx].split(/\s*=\s*/);
                if (aCouple.length > 1) {
                    oStorage[iKey = unescape(aCouple[0])] = unescape(aCouple[1]);
                    aKeys.push(iKey);
                }
            }
            return oStorage;
        };
        this.configurable = false;
        this.enumerable = true;
    })());
}

(function(){

    window.pageLoadSpeed = 10;
    window.debug = false;


    window.bazingaApp = window.bazingaApp || {};
	
	window.bazingaApp.models = window.bazingaApp.models || {};
	window.bazingaApp.collections = window.bazingaApp.collections || {};
	window.bazingaApp.views = window.bazingaApp.views || {};
    window.bazingaApp.views.components = window.bazingaApp.views.components || {};


}());

(function ($) {
    "use strict";


    var     translations= {

            },
            hashed = {
                'id': 'id',
                'preferred': 'pr',
                'data': 'da',
                'suspendData': 'sd',
                'role': 'ro',
                'currentScreen':'cs',
                'previousScreen' : 'ps',
                'score' : 'sc',
                'primaryScore' : 'x',
                'secondaryScore' : 'y',
                'tertiaryScore' : 'z',
                'gotoScreen' : 'gs',
                'count': 'co',
                'views':'vie',
                'completion': 'a',
                'page': 'pa',
                'not attempted': 'n',
                'passed': 'p',
                'failed': 'f',
                'completed': 'c',
                'incomplete': 'i',
                'courseScore' :'zs',
                'courseCompletion' : 'zc',
                'courseStatus' : 'crs',
                'progress' :'pg',
                'dialog-continue':'dcon',
                'dialog-complete':'dcom',
                'dialog-exit' : 'dex',
                'dialog-help' : 'dh',
                'courseData' : 'cd',
                'currentPool' : 'cp'
            },
        sanitizeMap  = {
            '&': '_%',
            '"': '_^',
            "'": '_*'
        },
        deSanitizeMap = _.invert(sanitizeMap),
        createSanitizeEscaper = function(map) {
            var escaper = function(match) {
                return map[match];
            };
            var source = '(?:' + _.keys(map).join('|') + ')';
            var testRegexp = RegExp(source);
            var replaceRegexp = RegExp(source, 'g');
            return function(string) {
                string = string == null ? '' : '' + string;
                return testRegexp.test(string) ? string.replace(replaceRegexp, escaper) : string;
            };
        },
        crunchSuspendData = function(){
            return function(data){
                var crunched = JSON.stringify(data);
                return (crunched.replace(/"/g,'*'));
            };
        },
        uncrunchSuspendData =  function(){
            return function(data){
                if(!_.isString(data)){return data;}
                var uncrunched = (data.replace(/\*/g,'"'));
                if(!uncrunched || uncrunched == ''){return {}}
                return JSON.parse(JSON.stringify(eval("(" + uncrunched + ")")));
            };
        };

    _.mixin({sanitizeScormString : createSanitizeEscaper(sanitizeMap)});
    _.mixin({deSanitizeScormString : createSanitizeEscaper(deSanitizeMap)});
    _.mixin({crunchSuspendData : crunchSuspendData()});
    _.mixin({uncrunchSuspendData : uncrunchSuspendData()});

    _.mixin({
        getKeyHash: function (hash) {
            return _.has(hashed,hash)? hashed[hash] : hash;
        }
    });

    _.mixin({
        translateText: function(text){
            return _.has(translations,text)? translations[text] : text;
        }
    });

    _.mixin({
        padNumber : function(n, width, z) {
            z = z || '0';
            n = n + '';
            width = width || 2;
            return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
        }
    });

    _.mixin({
        capitalize: function(string) {
            return string.charAt(0).toUpperCase() + string.substring(1).toLowerCase();
        }
    });


    // Usage:
    //
    // var obj = {
    //   a: {
    //     b: {
    //       c: {
    //         d: ['e', 'f', 'g']
    //       }
    //     }
    //   }
    // };
    //
    // Get deep value
    // _.deep(obj, 'a.b.c.d[2]'); // 'g'
    //
    // Set deep value
    // _.deep(obj, 'a.b.c.d[2]', 'george');
    //
    // _.deep(obj, 'a.b.c.d[2]'); // 'george'
    _.mixin({

        // Get/set the value of a nested property
        deep: function (obj, key, value) {

            var keys = key.replace(/\[(["']?)([^\1]+?)\1?\]/g, '.$2').replace(/^\./, '').split('.'),
                root,
                i = 0,
                n = keys.length;

            // Set deep value
            if (arguments.length > 2) {

                root = obj;
                n--;

                while (i < n) {
                    key = keys[i++];
                    obj = obj[key] = _.isObject(obj[key]) ? obj[key] : {};
                }

                obj[keys[i]] = value;

                value = root;

                // Get deep value
            } else {
                while ((obj = obj[keys[i++]]) != null && i < n) {};
                value = i < n ? void 0 : obj;
            }

            return value;
        }
    });


    _.mixin({
        pluckDeep: function (obj, key) {
            return _.map(obj, function (value) { return _.deep(value, key); });
        }
    });


    _.mixin({

        // Return a copy of an object containing all but the blacklisted properties.
        unpick: function (obj) {
            obj || (obj = {});
            return _.pick(obj, _.difference(_.keys(obj), _.flatten(Array.prototype.slice.call(arguments, 1))));
        }

    });
})(window.$ || window.JQuery);

if (!window.timeManager) {
    window.timeManager = function(){
        var self = this;
        self.hr = 0;
        self.min = 0;
        self.sec = 0;
        self.running = false;


        this.fromString = function(str){
            var split = (str || "00:00:00").split(":");

            self.hr = (parseInt(split[0]) || 0);
            self.min = (parseInt(split[1]) || 0);
            self.sec = (parseFloat(split[2]) || 0);
            console.log(self);
            return this;
        };


        this.toString  = function(){
            var self = this;
            return ( (_(self.hr).padNumber()) +":"+(_(self.min).padNumber(2))+":"+(_(self.sec).padNumber(2)));
        };

        this.addTo =  function(time){
            if(!time){return this.toString();}

            var self=this,
                split = new window.timeManager();
            split.fromString(time);

            split.sec += self.sec;

            if(split.sec > 60){
                split.min ++;
                split.sec = split.sec%60;
            }
            if(split.min > 60){
                split.hr ++;
                split.min = split.min%60;
            }
            return split;
        };

        this.doContinue =  function(){
            setTimeout(function(){
                self.sec ++;
                if(self.sec > 60){
                    self.min ++;
                    self.sec = self.sec%60;
                }
                if(self.min > 60){
                    self.hr ++;
                    self.min = self.min%60;
                }

                if(!self.running){
                    return false;
                }

                self.doContinue();
            },1000);
        };

        this.start = function(){
            self.running=true;
            self.doContinue();
            return this;
        };

        this.stop = function(){
            self.running = false;
        };

        this.isRunning = function(){
            return self.running;
        };

    };
}

(function($){
    "use strict";

    window.bazingaApp.models.scormLocalStorage = Backbone.Model.extend({
        'storage' : null,
        initailizeLocalStorageObject : function(){
        },
        reset : function(){
            localStorage.clear();
            return true;
        },
        remove : function(key){
            localStorage.removeItem(key);
            return true;
        },
        LMSInitialize: function(){
            this.initailizeLocalStorageObject();
        },
        LMSGetValue : function(key){
            return localStorage.getItem(key) || null;
        },
        LMSSetValue : function(key,value){
            this.set(key,value || null);

            localStorage.setItem(key,value);
        },
        LMSFinish : function(){
            //console.log("LMS finish called(LS) : ");
        },
        LMSCommit : function(){
            //console.log("LMS commit called(LS) : ");
            //this.save({key:value});
        },
        LMSGetLastError : function(){
            //console.log("LMS get error called(LS) : ");
            return '0';
        },
        LMSGetErrorString : function(){
            //console.log("LMS get error string called(LS) : ");
            return 'No Error';
        }
    });

})(window.$||window.JQuery);
(function($){
    "use strict";

    window.bazingaApp.models.timerModel = Backbone.Model.extend({
        defaults:  {
            secs : 0,
            mins : 0,
            hrs  : 0
        }
    });

})(window.$||window.JQuery);
(function($){
    "use strict";

    window.bazingaApp.models.dataModel = Backbone.Model.extend({
        defaults: {
            lesson_location:  '',
            student_name: '',
            lesson_status: 'not attempted',
            score_raw: null,
            score_min: 0,
            score_max: 100,
            suspend_data: '',
            comments: '',
            student_preference_audio: '0',
            student_preference_language: 'en_En',
            student_preference_speed: '0',
            student_preference_text: '0',
            mastery_score : null,
            'exit':'',
            'role':'learner',
            'courseVisits':0
        },
        changedSinceSync : [],
        dirty  : false,


        initialize : function(){
            var self=this;
            this.hash = _.extend({},{
                'lesson_location':'cmi.core.lesson_location',
                'student_name' :  'cmi.core.student_name',
                'credit':'cmi.core.credit',
                'lesson_status':'cmi.core.lesson_status',
                'entry':'cmi.core.entry',
                'score_raw':'cmi.core.score.raw',
                'score_min':'cmi.core.score.min',
                'score_max':'cmi.core.score.max',
                'suspend_data':'cmi.suspend_data',
                'launch_data':'cmi.core.launch_data',
                'mastery_score':'cmi.student_data.mastery_score',
                'comments':'cmi.core.comments',
                'comments_from_lms':'cmi.core.comments_from_lms',
                'student_preference_audio':'cmi.student_preference.audio',
                'student_preference_language':'cmi.student_preference.language',
                'student_preference_speed':'cmi.student_preference.speed',
                'student_preference_text':'cmi.student_preference.text'
            });

            _.bindAll(this,'upload','commit');
            this.on("change",this.upload ,self);
        },

        autoSave : function(){
            var self = this;
            var check = function(){
                setTimeout(function(){
                    self.upload(true).always(check);
                    console.log("Data autosaved");
                },10000);
            };
            check();
        },

        fetch : function(options){
            var self=this,
                API= self.API,
                keys,
                deferred= $.Deferred(),
                val;

            if(!API){return false;}

            setTimeout(function(){
                //Start loading all the values
                _.each(self.hash,function(keyhash,key){
                    val = API.LMSGetValue(keyhash);
                    self.set(key,val,{silent: true});
                });
                deferred.resolve(self);
            },500);
            return deferred.promise();
        },
        isDirty : function(){return this.dirty;},
        getChangedSinceSync : function(){return this.changedSinceSync;},
        commit : function(){
            var self=this,
                API= self.API;

            if(!API) return false;
            console.log("Changed called -> committing");
            API.LMSCommit("");
            return true;
        },
        upload : function (commit) {
            //options.data = _.pick(this.attributes, 'foo', 'bar', 'baz');
            var self=this,
                API= self.API,
                deferred= $.Deferred(),
                val,
                changed,
                hashedKey;

            commit = commit || false;
            if(!API){
                setTimeout(function(){deferred.fail();},10);
            }
            else{
                setTimeout(function(){
                    changed = self.changed;
                    if(self.dirty){

                        console.log("-- UPLOADING VALUE --");
                        _.each(self.changedSinceSync, function(key){
                             val = self.get(key);
                             if(_.has(self.hash,key)){
                                 hashedKey  = self.hash[key];
                                 API.LMSSetValue(hashedKey,val);
                                 console.log(hashedKey +" "+val);
                             }
                        });
                        console.log("-- UPLOAD COMPLETE --");

                        self.changedSinceSync = [];
                        self.dirty = false;
                    }
                    if(commit){
                        API.LMSCommit("");
                    }

                    deferred.resolve();
                },500);
            }
            return deferred.promise();
        },

        setData : function(){
            var self=this,
                argumentsArray = [].slice.apply(arguments),
                attr = argumentsArray[0],
                value = argumentsArray[1],
                change = function(attr,value){
                    self.set((""+attr+""),value);
                    if(_.indexOf(self.changedSinceSync,(""+attr+"")) == -1){
                        self.changedSinceSync.push((""+attr+""));
                        self.dirty = true;
                    }
                };



            if(!attr){return false;}

            if(typeof attr == 'object'){
                return _.each(attr,this.setData);
            }
            else{
                switch(attr){
                    case "suspend_data":
                    case "lesson_location":
                    case "comments":
                        //allow write freely
                        break;
                    case "score_min":
                        var max=self.getData('score_max');
                        if( (_.isNumber(value) && value)  && value >= 0 && value <=100 && (!_.isNumber(max) || (value < max))) {
                            change(attr,value);

                            return true;
                        }

                        change(attr,0);
                        break;

                    case "score_max":
                        var min=self.getData('score_min');

                        if( (_.isNumber(value) && value)  && value > 0 && value <=100 && (!_.isNumber(min) || (value > min))) {
                            change(attr,value);
                            return true;
                        }
                        change(attr,100);
                        return false;
                        break;

                    case 'score_raw':
                        if(_.isNumber(value)) {
                            var currentStatus=self.getData('lesson_status');


//                            if(['passed','completed'].indexOf(currentStatus) > -1){
//                                //Do not update score after the course is completed
//                                return false;
//                            }


                            value =(function(){
                                var min=self.getData('score_min'),
                                    max=self.getData('score_max');

                                if(value === null || value === ''){return null;}

                                if(!_.isNumber(min)||!_.isNumber(max)){
                                    //ceil off the value
                                    return (value <= 0) ? 0 : ((value >= 100) ? 100 : value);
                                }

                                if(min >= max ){return value;}
                                value = (value <= min) ? min : ((value >= max) ? max : value);

                                //convert to percentage
                                return ((value-min)/(max-min))*100;
                            })();
                        }

                        var currentScore = self.getData('score_raw');
                        if(_.isNumber(currentScore) && (!_.isNumber(value) || currentScore > value)){
                            //If the current score is greater than the attempted set then dont update
                            return false;
                        }


                        var masteryScore = self.getData('mastery_score');

                        if(!_.isNumber(value)){
                            //Bad hack . have to return false but then have ot set values as well
                            if(!this.get(attr)){
                                this.set((""+attr+""),null);
                            }
                            return false;
                        }


                        if(masteryScore){
                            if(value < masteryScore){
                                this.setData('lesson_status','failed');
                            }
                            else if(value >= masteryScore){
                                this.setData('lesson_status','passed');
                            }
                        }


                        break
                    case 'lesson_status':
                        if(['not attempted','incomplete','completed','passed','failed'].indexOf(value) == -1){
                            return false;
                        }
                        var currentStatus=self.getData('lesson_status');

                        if(currentStatus == 'passed'){return false;}
                        else if(currentStatus=='completed' && value !='passed'){return false;}
                        else if(currentStatus=='failed' && ['passed','completed'].indexOf(value) == -1){return false;}
                        else if(currentStatus=='incomplete' && (value=='not attempted' || value=='incomplete')){return false;}
                        else if(currentStatus == value){return false;}
                        break;

                    case 'total_time':
                    case 'session_time':
                        //do some testing around total time
                        break;

                    case 'entry':
                        if(['ab-initio','resume'].indexOf(value) == -1){
                            return false;
                        }
                        break;

                    case 'exit':
                        if(['time-out','suspend','logout',''].indexOf(value) == -1){
                            return false;
                        }
                        break;


                    case 'launch_data':
                    case 'lesson_mode':
                    case 'student_name':
                    case 'comments_from_lms':
                    case 'name':
                    case 'credit':
                    case 'mastery_score':
                        //read only values
                        return false;

                    case 'student_preference_audio':
                        if(value !== '' && (!_.isNumber(value) || -1 > value || 100 < value)){
                            return false;
                        }
                        break;
                    case 'student_preference_speed':
                        if(value !== '' && (!_.isNumber(value) || -100 > value || 100 < value)){
                            return false;
                        }
                        break;
                    case 'student_preference_text':
                        if(value !== '' && (!_.isNumber(value) || -1 > value || 1 < value)){
                            return false;
                        }
                        break;
                    default:
                        break;
                }

                change(attr,value);
            }

            return true;
        },
        getData : function(){
            var self=this,
                argumentsArray = [].slice.apply(arguments),
                attr = argumentsArray[0];

            /*@todo  : handle interactions*/
            return this.get(attr);
        },

        setAPI : function(API){
            this.API = API;
            return this;
        },
        getAPI : function(){
            return this.API;
        },
        setNavModel : function(navModel){
            this.navModel = navModel;
            return this;
        },
        getNavModel : function(){
            return this.navModel || null;
        },
        //Utility methods
        setLessonLocation: function(lesson_location){
            return this.setData('lesson_location',lesson_location);
        },

        getLessonLocation : function(){
            return this.getData('lesson_location');
        },

        setLessonStatus: function(lesson_status){
            return this.setData('lesson_status',lesson_status);
        },

        getLessonStatus : function(){
            return this.getData('lesson_status');
        },
        setScore : function(score){
            return this.setData('score_raw',score);
        },
        getScore : function(){
            return this.getData('score_raw');
        },
        getSuspendData : function(){
            var d =  this.getData('suspend_data');
            if(d==undefined || d == 'undefined' || d == null || !d){
                d = '';
            }
            return d;
        },
        forceSetSuspendData : function(suspend_data){
            var self=this,
                API= self.API;

            this.setData('suspend_data',suspend_data);
            API.LMSSetValue('cmi.suspend_data',this.get('suspend_data'));
            return this;
        },
        setSuspendData: function(suspend_data,force){
            this.setData('suspend_data',suspend_data);
            return this;
        },
        getMasteryScore : function(){
            return this.getData('mastery_score');
        },
        setAudioPreference : function(audioPreference){
            this.set('student_preference_audio',audioPreference);
            return this;
        },
        getAudioPreference : function(){
            return this.get('student_preference_audio');
        },
        getSpeedPreference : function(){
            return this.get('student_preference_speed');
        },
        setSpeedPreference : function(speedPreference){
            this.set('student_preference_speed',speedPreference);
            return this;
        },
        getLanguagePreference : function(){
            return this.get('student_preference_language');
        },
        setLanguagePreference : function(languagePreference){
            this.set('student_preference_language',languagePreference);
            return this;
        },
        getTextPreference : function(){
            return this.get('student_preference_text');
        },
        setTextPreference : function(textPreference){
            this.set('student_preference_text',textPreference);
            return this;
        },
        incrementScore : function(increment){
            var raw = this.getScore();
            this.setScore(raw+increment);
        },
        setComments : function(comments){
            return this.setData('comments',comments);
        },
        getComments : function(){
            return this.getData('comments');
        }

    });

})(window.$||window.JQuery);
(function ($) {
    "use strict";

    window.bazingaApp.models.scormDataModel = Backbone.Model.extend({
        defaults: {
            "initialized": false,
            "API": null,
            "status": "new",
            "lastError": "0",
            "lastErrorDescription": "No Error",
            "autoCommit": false
        },

        sessionTime  : new window.timeManager(),

        initialize: function () {
            var API,
                self = this;
            if (!this.get('API')) {
                this.findAPI();
            }

            this.initializeSCO();
            API = this.get('API');

            self.sessionTime
                .fromString("00:00:00.00")
                .start();


            self.totalTime = API.LMSGetValue("cmi.core.total_time");
            self.totalTime = self.totalTime || "0000:00:00.00";
            return this;
        },

        commit: function () {
            var self = context || this,
                API = self.get('API');

            API.LMSCommit("");
            return this;
        },

        setData: function (data) {
            this.data = data;
            return this;
        },

        findAPI: function () {
            var lwindow = window,
                loops = 10,
                API = null,
                self = this,
                locateAPI = function (win) {
                    while ((win.API == null) &&
                        (win.parent != null) &&
                        (win.parent != win)) {
                        // increment the number of findAPITries
                        loops++;
                        // Note: 20 is an arbitrary number, but should be more than sufficient
                        if (loops > 50) {
                            alert("Error finding API -- too deeply nested.");
                            return null;
                        }

                        // set the variable that represents the window being
                        // being searched to be the parent of the current window
                        // then search for the API again
                        win = win.parent;
                    }
                    return win.API;
                };

            API = locateAPI(lwindow);

            if (!API) {
                console.log("API was not located. A local storage system will be initialized");
                API = new window.bazingaApp.models.scormLocalStorage();
            }
            else{
                console.log("API was successfully located");
            }

            this.set('API', API);
        },


        setValue: function (key, value, forceCommit, context) {
            var self = context || this,
                API = self.get('API'),
                returnValue = null;

            forceCommit = forceCommit || false;

            if (!API || !key) {
                console.error("Unable to get Data . API Not initialized or the key is undefined");
                return null;
            }

            API.LMSSetValue(key, value);
            self.scormError();

            if (self.get('autoCommit') || forceCommit) {
                API.LMSCommit("");
            }

            return self;
        },

        getValue: function (key, context) {
            var self = context || this,
                API = self.get('API'),
                returnValue = null;
            if (!API || !key) {
                console.error("Unable to get Data . API Not initialized or the key is undefined");
                return null;
            }

            returnValue = API.LMSGetValue(key);
            self.scormError();

            return returnValue;
        },
        initializeSCO: function (context) {
            var self = context || this,
                API = self.get('API');

            if (self.get('initialized')) {
                //pre-initialized . then ignore.
                console.log("API was previously initialized");
                return false
            };


            self.set('initialized', (API && typeof API.LMSInitialize == 'function' && API.LMSInitialize("")));
            console.log("API initialized "+ self.get('initialized'));

            var status = self.getValue('cmi.core.lesson_status');
            if(status == 'not attempted'){self.setValue('cmi.core.lesson_status','incomplete');}

            //reset the error codes
            self.scormError();
            self.set('status', 'active');
        },

        switchTimers : function(context){
            var self = context || this,
                API = self.get('API');
            API.LMSSetValue("cmi.core.session_time",self.sessionTime.toString());

            console.log("Seting session time "+self.sessionTime.toString());
            self.sessionTime.stop();
            API.LMSCommit("");
        },
        finishSCO: function (context) {
            var self = context || this,
                API = self.get('API'),
                deferred = $.Deferred();

            if(window.bazingaApp.preloader){window.bazingaApp.preloader.setPercentage(0).enable();}



            console.log("TOTAL TIME "+self.totalTime.toString() + ' ---> ' + API.LMSGetValue("cmi.core.total_time"));

            if (!self.get('initialized')) {
                console.warn("Finish sco called before initialization");
            }
            else{
                if (API && typeof API.LMSFinish == 'function' && API.LMSFinish("")) {
                    API.LMSFinish("");
                }
            }




            if(self.data){
                self.data
                    .upload()
                    .always(function () {
                        API.LMSCommit("");
                        self.set('initialized', false);
                        self.set('status', 'inactive');
                        console.log("All data synced. LMS commit called. shutting down");
                    });
            }
            else{
                API.LMSCommit("");
                self.set('initialized', false);
                self.set('status', 'inactive');
                console.log("All data synced. LMS commit called. shutting down");
            }

            if(window.bazingaApp.preloader){window.bazingaApp.preloader.incrementTo(75);}
            setTimeout(function(){
                deferred.resolve();
            },2000);
            return deferred.promise();
        },

        scormError: function (context) {
            var self = (context || this),
                API, lastErrorNumber;

            API = self.get('API');
            if (!API) {
                console.warn("LMS API not located");
                return false;
            }

            if (!self.get('initialized')) {
                console.warn("LMS API NOT initialized");
                return false;
            }

            lastErrorNumber = API.LMSGetLastError();

            self.set('lastError', lastErrorNumber);
            self.set('lastErrorDescription', API.LMSGetErrorString(lastErrorNumber))

            return self;
        }
    });

})(window.$ || window.JQuery);
(function ($) {
    "use strict";
    window.quedSuspendDataUpdate = false; 

    /**
     * Get the query parameters from the url
     * @param qs
     * @returns {{}}
     */
    window.getQueryParams = function(qs) {  
        qs = qs.split('+').join(' ');

        var params = {},
            tokens,
            re = /[?&]?([^=]+)=([^&]*)/g;

        while (tokens = re.exec(qs)) {
            params[decodeURIComponent(tokens[1])] = decodeURIComponent(tokens[2]);
        }

        return params;
    }


    /**
     * Model for the current page data.
     * Every page can have 4 parameters : count / status / data / score
     * @type {*}
     */
    window.bazingaApp.models.pageData = Backbone.Model.extend({
        initialize: function () {
            _.bindAll(this, 'incrementCount', 'setStatus', 'setData', 'setScore', 'setCount');
        },
        incrementCount: function (options) {
            options = options || {};
            this.set(_('count').getKeyHash(), ((this.get(_('count').getKeyHash())) || 0) + 1);
            return this;
        },
        setStatus: function (status, options) {
            options = options || {};
            this.set(_('completion').getKeyHash(), _(status).getKeyHash(), options);
            if(_.has(this.changed,_('completion').getKeyHash())){
                window.was_page_completed_on_load = true;
            }
            else{
                window.was_page_completed_on_load = false;
            }
            return this;
        },
        setData: function (data, options) {
            options = options || {};
            this.set(_('data').getKeyHash(), data, options);
            return this;
        },
        setCount: function (count, options) {
            options = options || {};
            this.set(_('count').getKeyHash(), count, options);
            return this;
        },
        setScore: function (score, options) {
            options = options || {};
            score = score || null;
            this.set(_('score').getKeyHash(), score, options);
            return this;
        },
        getScore: function () {
            return this.get(_('score').getKeyHash()) || null;
        },
        getCompletion: function () {
            return _(this.get(_('completion').getKeyHash())).getKeyHash();
        },
        setCompletion: function (status) {
            return this.setStatus(status);
        },
        getData: function () {
            return this.get(_('data').getKeyHash()) || null;
        },
        getCount: function () {
            return this.get(_('count').getKeyHash()) || 0;
        }
    });

    /**
     * Page navigation
     * @type {*}
     */
    window.bazingaApp.models.pageNav = Backbone.Model.extend({});

    /**
     * Core navigation model that controls all functionalities of the course.
     *
     * @type {*}
     */
    window.bazingaApp.models.navigationModel = Backbone.Model.extend({
        dataModel: null,

        /**
         * List of all parameters that can be set using the navigation model.
         * These parameters are hashed using the getKeyHash() mixin
         */
        defaults: {
            preferred: null,
            role: "default",
            count: 0,
            data: null,
            suspendData: null,
            currentScreen: null,
            previousScreen: null,
            currentPool : 1,
            courseStatus : 'na',
            primaryScore:null,
            secondaryScore :null,
            tertiaryScore : null,
            courseData : '',
            nav: null,
            lu : null
        },

        conditional : {
            "lockables" :[],
            "locked" : [],
            "unlocked" : []
        },
        /**
         * These parameters are not saved into the suspend Data but used extensively inside the current model
         * Setting these never fires a 'set' or 'get' call
         */
        values: {
            allPages: [],
            suspendDataIdx: null,
            allDialogs : [],
            isDialogScreen : false
        },
        models: {
            currentSuspendData: null
        },


        initialize: function () {
            var self = this;
            _.bindAll(this, 'loadFrom', 'crunchForRole', 'retrieveSuspendData',
                'handshake', 'setUpCurrentScreen', 'suspendDataChanged', 'updateSuspendData',
                'pageInitializers', 'setSuspendDataIndices', 'getNavForScreenByRole',
                'setPageCompletion', 'setPageData', 'setInfo', 'setPageScore' , 'setCourseScoreFrom' ,
                'setCourseScore','setCourseStatus', 'reattemptScreen','setCurrentPool','getCurrentPool');

            this.on("change:role", this.retrieveSuspendData, self);
            this.on("change", this.suspendDataChanged, self);
        },

        /**
         * The event handler everytime data changes within the page.
         * The suspendDataChanged function will create a new suspendDataString and commit it to the LMS
         * @param __force
         * @returns {*}
         */
        suspendDataChanged: function (__force) {
            var self = this,
                suspendData = self.retrieveSuspendData(),
                dataModel = self.dataModel,
                deferred = $.Deferred();

            __force = __force || false;


            setTimeout(function () {
                if (__force) {
                    dataModel.forceSetSuspendData(suspendData);
                }
                else {
                    dataModel.setSuspendData(suspendData);
                }
                dataModel.commit();

                window.updateOtherData = false;
                deferred.resolve();
                self.trigger("initialized:suspendData", self);
            }, window.pageLoadSpeed);


            return deferred.promise();
        },

        /**
         * STEP 1
         * Load the josn file and set up the preferred data
         * @param jsonFile
         * @returns {*}
         */
        load: function (jsonFile) {
            var deferred = $.Deferred(),
                self = this,
                hashed = {
                    'completion' : _('completion').getKeyHash(),
                    'completed' : _('completed').getKeyHash(),
                    'passed' : _('passed').getKeyHash(),
                    'incomplete' : _('incomplete').getKeyHash(),
                    'failed' : _('failed').getKeyHash(),
                    'not_attempted' : _('not attempted').getKeyHash(),
                };

            $.getJSON(jsonFile, function (data) {
                self.set('preferred', data, {silent: true});
                self.handshake()
                    .always(function () {
                        self.recalculateCompletion()
                            .always(function(){
                                var i = self.get('courseStatus');


                                var suspendData = self.get('suspendData');
                                _.each(self.conditional.lockables, function(lockables,tgtId){
                                    var locked = true,
                                        suspend,
                                        status = hashed.not_attempted,
                                        lockedStatus =[]; 

                                    _.map(lockables.lockedIf, function(lockable){
                                        if(!lockable.id){return true;}

                                        console.log("Lockable identified "+lockable.id + " .tgt = "+tgtId);
                                        var inverse = false;
                                        suspend = _.first((_.where(suspendData,{id:lockable.id})));
                                        status = hashed.not_attempted;

                                        if(suspend && !_.isEmpty(suspend) && _.has(suspend,hashed.completion)){
                                            status = suspend[hashed.completion];
                                        }

                                        if(_.has(lockable,'condition') && lockable.condition=="!"){
                                            inverse = true;
                                        }

                                        if([hashed.not_attempted,hashed.incomplete,hashed.failed].indexOf(status) > -1){
                                            //not completed the page
                                            lockedStatus.push(inverse);
                                        }
                                        else{
                                            //Page is complete 
                                            lockedStatus.push(!inverse);
                                        }

                                    });
                                    

                                    console.log(lockedStatus);
                                    locked = (-1 != lockedStatus.indexOf(true));
                                    lockables.locked = locked;
                                    self.conditional.lockables[tgtId] = lockables;
                                });


                                //resolve and start application
                                deferred.resolve(self);
                            });
                    });
            });
            return deferred.promise();
        },

        /**
         * STEP 2
         * Set up handshake between the data models
         * This will link the data model to the navigation model and initialize the navigation
         * @returns {*}
         */
        handshake: function () {
            var deferred = $.Deferred(),
                self = this,
                dataModel = self.dataModel,
                hashed = {
                    'completion' : _('completion').getKeyHash(),
                    'completed' : _('completed').getKeyHash(),
                    'passed' : _('passed').getKeyHash(),
                    'incomplete' : _('incomplete').getKeyHash(),
                    'failed' : _('failed').getKeyHash(),
                    'not_attempted' : _('not attempted').getKeyHash()
                };

            console.log("Hand shake");
            self.trigger("initialize:handshake", self);
            setTimeout(function () {
                self.trigger("initialize:suspendData", self);
                if (!dataModel) {
                    deferred.fail();
                    return false;
                }

                var suspendData = dataModel.getSuspendData(),
                    preferredForRole;

                /**
                 * Step 1 : Load from suspend data :
                 *          Retrieve the suspend data string and parse it .
                 *
                 * Step 2 : crunchForRole
                 *          Depending on the role and existing data a navigation structure will be populated
                 *          If suspenddata preexists then the navigation will be retrieved from the preexisting data
                 *
                 * Step 3 : setUpCurrentScreen
                 *          Reinitialize the page values to reflect changes on the current screen .
                 *
                 * Step 4 : pageInitializers
                 *          Initialize page avalues within this page
                 *
                 * Step 5 : If the page is the first page launched then either navigate to the dialog continue / home or the current page
                 *
                 *
                 * Step 6 : resolve  the promise object
                 */
                self.loadFrom((suspendData || ''), self)
                    .crunchForRole(self.get('preferred'))
                    .always(function (data) {
                        //Set up the preferred navigation
                        self.set('preferred', data, {silent: true});

                        if (!self.get('suspendData')) {
                            console.error("Error initializing suspend data");
                        }

                        self.setUpCurrentScreen();
                        self.pageInitializers()
                            .always(function () {
                                console.log("[DBG:] Page is now initialized="+self.get('courseStatus'));

                                var currentScreen,
                                    previousScreen,
                                    role,
                                    nav,
                                    pageNav,
                                    goto,
                                    nextNav,
                                    resolve=false,

                                    //Check if the course is complete
                                    isCourseComplete = (function(){
                                        if(window.was_page_completed_on_load){
                                            console.log("Page was only just completed + course status " + self.get('courseStatus'));
                                            return false;
                                        }


                                        return (([hashed.completed,hashed.passed].indexOf(self.get('courseStatus'))) > -1);
                                    })(),
                                    isCurrentComplete;


                                    if( isCourseComplete || window.localStorage.getItem('firstLaunch')){

                                        console.log("Course is complete")
                                        window.localStorage.removeItem('firstLaunch');

                                        //check to see if a dialog needs to be shown.
                                         currentScreen = self.get('currentScreen');
                                         previousScreen = self.get('previousScreen');
                                         role = self.get('role') || 'default';


                                        if(!isCourseComplete && !previousScreen){
                                            //Launch for the first time .  The course is incomplete
                                            resolve = true;
                                        }
                                        else{
                                            currentScreen = currentScreen || 'home';
                                            nav = self.getNavForScreenByRole(role,previousScreen);

                                            if(isCourseComplete){
                                                console.log("Course Config ");
                                                if(self.courseConfig.allow_continue_after_complete){
                                                    //Do not lock the course on complete. Continue
                                                    resolve = true;
                                                    debugger;
                                                }
                                                else{
                                                    //force the course to go to the dcom
                                                    goto = self.courseConfig.default_complete || 'dcom';

                                                    isCurrentComplete = (function(){


                                                        var location = window.location.pathname,
                                                            locationHref = (location.split("/")).pop(),
                                                            dialogNav;

                                                        if(["dcom"].indexOf(goto) > -1){
                                                            //send the course score on dialog complete
                                                            self.setCourseScoreFrom('primary');
                                                            dialogNav = self.getDialog(goto);
                                                        }
                                                        else{
                                                            dialogNav = self.getNavForScreen(goto);
                                                        }


                                                        return (locationHref === dialogNav.resource)
                                                    })();

                                                    if(isCurrentComplete){resolve  = true;}
                                                }

                                            }
                                            else{
                                                goto = nav.reenter || self.courseConfig.default_continue || 'dcon';
                                            }




                                            if(window.debug || resolve){
                                                resolve = true;
                                            }
                                            else{

                                                if((goto && goto!='prevent')){
                                                    //go to a particular screen or dialog
                                                    if(["dcom","dcon","dex","dh","dr"].indexOf(goto) > -1){
                                                        nextNav = self.getDialog(goto);
                                                    }
                                                    else{
                                                        nextNav = self.getNavForScreen(goto);
                                                    }

                                                    if(nextNav && nextNav.resource){
                                                        window.location.replace(nextNav.resource);
                                                    }
                                                    else{
                                                        //do nothing skip
                                                        resolve=true;
                                                    }
                                                }
                                                else if(currentScreen !== previousScreen){

                                                    pageNav = self.getNavForScreen(previousScreen);

                                                    if( pageNav &&  pageNav.resource){
                                                        window.location.replace(pageNav.resource);
                                                    }
                                                    else{
                                                        //do nothing skip
                                                        resolve=true;
                                                    }
                                                }
                                                else{
                                                    resolve=true;
                                                }
                                            }

                                        }
                                    }
                                    else{

                                        resolve=true;
                                    }

                                    if(resolve){deferred.resolve();}

                            });

                    });
            }, window.pageLoadSpeed);

            return deferred.promise();
        },


        /**
         * Load the current data model from the suspend data string
         * @param suspendData
         * @param context
         * @returns {*}
         */
        loadFrom: function (suspendData, context) {
            var self = context || this,
                suspendDataJson = _(suspendData).uncrunchSuspendData(),
                attributes = context.attributes,
                attributeKeys = _.keys(attributes);


            _.each(attributeKeys, function (key) {

                var keyHash = _(key).getKeyHash();
                switch (key) {
                    case 'preferred':
                        break;
                    case 'suspendData':
                        var dataJson = {},
                            pageHash = _('page').getKeyHash(),
                            preventTracking = false;

                        if (_.has(self.courseConfig,'prevent_tracking') && self.courseConfig.prevent_tracking == true){preventTracking = true;}

                        if (!_.isEmpty(suspendDataJson) && !preventTracking) {
                            //reset the key from the suspend data
                            if (_.has(suspendDataJson, pageHash)) {
                                dataJson = suspendDataJson[pageHash];
                            }

                            self.set('suspendData', dataJson, {silent: true});
                        }
                        break;
                    case 'nav':
                        //reset the key from the suspend data
                        if (_.has(suspendDataJson, keyHash)) {
                            context.set(key, suspendDataJson[keyHash], {silent: true});
                        }
                        ;
                        break;

                    default:
                        //reset the key from the suspend data
                        if (_.has(suspendDataJson, keyHash)) {
                            context.set(key, suspendDataJson[keyHash], {silent: true});
                        }
                        ;
                        break
                }
            });
            return self;
        },

        /**
         * Set up the navigation for the role
         * @param data
         * @returns {*}
         */
        crunchForRole: function (data) {

            var role = this.get('role') || 'default',
                currentNav = this.get('nav') || null,
                deferred = $.Deferred(),
                self = this,
                cp  = self.get('currentPool'),

                /**
                 * When the config timestame is greater than the timestamp on file
                 * Reload the config / navigation
                 * This allows us to make sure the suspendData string is in sync with the navigation
                 *
                 * @Note : changin the timestamp will reset the suspendData string
                 */
                isConfigUpdated = (function(){

                    //Compare date time with existing date tiem
                    var d = new Date(),
                        d2,
                        t = d.getTime(),
                        t2,
                        lu = self.get('lu'),
                        clu;

                    if (!self.get('suspendData') || !self.courseConfig || !lu) {
                        //time was never recorded . record it now
                        self.set('lu',t,{silent:true});
                        return false;
                    }

                    clu = self.courseConfig.last_updated;

                    if(!clu){
                        self.set('lu',t,{silent:true});
                        return false;
                    }

                    d2 = new Date(clu);
                    t2 = d2.getTime();
                    self.set('lu',t2);



                    if(t2 > lu){
                        return true;
                    }
                    return false;
                }()),

                /**
                 * This  is the access control within the nav controller.
                 * The page filter loops through each page and make sure the page values are parsed
                 * Also it creates easy access variables within the application
                 */
                pageFilter = function (pages, cNav, parent) {
                    //map all sub pages
                    _.map(pages, function (page) {
                        //set up the page bread crumbs
                        page.crumbs = page.crumbs || [];
                        if(parent){
                            page.crumbs.push(parent);
                        }

                        //if the page has no items then initialize hte page
                        if (!_.has(page, 'items')) {
                            if(page.structure == 'group'){return page;}
                            page.structure = page.id;
                            page.parent = parent || null;
                            return page;
                        }

                        /**
                         * Step 1 : recursively filter the page items to make sure only  items with proper access levels can be accessed by the role
                         * Step 2 : randomize
                         *          randomize is dependent on the group level randomize options . Each nav key can be set to 3 different options
                         *                ">>" : Will randomize then reset the nav key with the id of the next page in the randomized list.
                         *                "<<" : Will randomize then reset the nav key with the id of the previous page in the randomized list.
                         *                "<id>" : This will set the nav key of the page to the id mentioned
                         *                "*" : this will inherit the defined nav properties for the page
                         *
                         * @type {*}
                         */
                        var items = pageFilter(page.items, (cNav ? (_.has(cNav, page.id) ? cNav[page.id] : null) : null), page.id),
                             randomize = (page.type == 'group' && _.has(page, 'randomize')) ? page.randomize : false,
                            __allIds = _.pluck(items, 'id'),
                            __static, __dynamic, itemList = [],
                            __after = {
                                "first": {
                                    "back": "home",
                                    "next": ">>",
                                    "complete": "home",
                                    "passed": "home",
                                    "failed": "home"
                                },
                                "last": {
                                    "back": "<<",
                                    "next": "home",
                                    "complete": "home",
                                    "passed": "home",
                                    "failed": "home"
                                },
                                "others": {
                                    "back": "<<",
                                    "next": ">>",
                                    "complete": "*",
                                    "passed": "*",
                                    "failed": "*"
                                }
                            };

                        //get a list of all pages (if previously set then get the right list.
                        if (_.has(cNav, page.id) && typeof cNav[page.id] === 'object') {
                            //@todo clean up the code
                            //since it is an object it automatically reorders numbers. so we switch the ordering to keys and let it sort then pick the values :D
                            //hack hack hack .SCREAMS HACK!! @sundar
                            __allIds = _.values(_.object(_.values(cNav[page.id]), _.keys(cNav[page.id])));
                        }
                        else if (randomize) {

                            //randomize the items
                            __static = page.randomize.__static || [];
                            __dynamic = _.shuffle(_.difference(__allIds, __static));

                            __allIds = _.map(_.clone(__allIds), function (id) {
                                if (_.indexOf(__static, id) > -1) {
                                    return id;
                                }
                                var dyn_id = __dynamic.pop();
                                return  dyn_id;
                            });
                        }


                        if (_.has(page.randomize, '__after')) {
                            __after = _.extend(__after, ((randomize && _.has(page.randomize, '__after')) ? page.randomize.__after : {}));
                        }


                        _.each(__allIds, function (value, key) {
                            var itemForId = _.find(items, function (it) {
                                    return it.id == value;
                                }),
                                nav ,
                                currentIdx = itemForId ? (_.indexOf(__allIds, itemForId.id)) : -1,
                                nextOrBack,
                                navByRoleForItem = itemForId.role[role].nav;

                            //This was a corrupted navigation file. Previous suspend data has since been changed
                            if(!itemForId){return true;}

                            //@todo :: figure out a better way of doing this
                            //currently the first screen always has a back to home. last screen always have next to home
                            if (key == 0) {
                                nav = __after.first;
                            }
                            else if (key == (__allIds.length - 1)) {
                                nav = __after.last;
                            }
                            else {
                                nav = __after.others;
                            }


                            _.each(nav, function (v, k) {
                                if (v == '*') {
                                    //Do nothing and take the defaults
                                    return true;
                                }
                                else if (v == '<<') {
                                    //Auto calculate back or next
                                    if (currentIdx == 0 || currentIdx == -1) {
                                        nextOrBack = 'home';
                                    }
                                    else {
                                        nextOrBack = (function () {
                                            if(currentIdx == 0){
                                                return 'home';
                                            }

                                            /**
                                             * This has been improved to add in the pool
                                             * When a back or next value is returned, the returned value is checked to verify
                                             * if the pools of the returned value matches with the currently selected pool
                                             * Or the loop continues till the match occurs
                                             *
                                             * @type {boolean}
                                             */
                                            var stop = false,
                                                cIdx = currentIdx - 1,
                                                idx,
                                                itForBack,
                                                retValue = 'home';

                                            while (stop === false){
                                                idx = __allIds[cIdx];
                                                itForBack = _.find(items, function (it) {
                                                    return it.id == idx;
                                                });

                                                if(!itForBack || !itForBack.id || !cp || !page.track || !page.track.pool  ){
                                                    if(itForBack && itForBack.id){
                                                        retValue = itForBack.id;
                                                    }
                                                    stop = true;
                                                }
                                                else{
                                                    if((page.track.pool+"") !== (cp+"")){
                                                        cIdx -= 1;
                                                        if(cIdx < 0 ){stop = true;}
                                                    }
                                                    else{
                                                        stop = true;
                                                        retValue = itForBack.id;
                                                    }
                                                }
                                            }


                                            return retValue;
                                        })();
                                    }

                                    console.log("Back  for "+value +" located as "+nextOrBack);
                                    itemForId.role[role].nav[k] = (nextOrBack || 'home');
                                    return true;
                                }
                                else if (v == '>>') {
                                    //Auto calculate back or next
                                    if (currentIdx == __allIds.length - 1) {
                                        nextOrBack = 'home';
                                    }
                                    else {
                                        nextOrBack = (function () {
                                            if(currentIdx >=  __allIds.length - 1){
                                                return 'home';
                                            }

                                            /**
                                             * This has been improved to add in the pool
                                             * When a back or next value is returned, the returned value is checked to verify
                                             * if the pools of the returned value matches with the currently selected pool
                                             * Or the loop continues till the match occurs
                                             *
                                             * @type {boolean}
                                             */
                                            var stop = false,
                                                cIdx = currentIdx + 1,
                                                idx,
                                                itForNext,
                                                retValue = 'home';

                                            while (stop === false){
                                                idx = __allIds[cIdx];
                                                itForNext = _.find(items, function (it) {return it.id == idx;});

                                                if(!itForNext || !itForNext.id || !cp || !page.track || !page.track.pool  ){
                                                    if(itForNext && itForNext.id){
                                                        retValue = itForNext.id;
                                                    }

                                                    stop = true;
                                                }
                                                else{
                                                    if((page.track.pool+"") !== (cp+"")){
                                                        cIdx += 1;
                                                        if(cIdx >= (__allIds.length - 1)){stop = true;}
                                                    }
                                                    else{
                                                        stop = true;
                                                        retValue = itForNext.id;
                                                    }
                                                }
                                            }


                                            return retValue;
                                        })();
                                    }
                                    console.log("Next  for "+value +" located as "+nextOrBack);
                                    itemForId.role[role].nav[k] = (nextOrBack || 'home');
                                    return true;
                                }

                                //Set the constant page as the key
                                navByRoleForItem[k] = v;
                                return true;
                            });

                            itemForId.role[role].nav = navByRoleForItem;
                            itemList.push(itemForId);
                        });
                        page.items = itemList;

                        var i = 0;

                        _.map(itemList, function (it) {
                            if (typeof it.structure !== 'object') {
                                it.structure = i++;
                            }
                            return it;
                        });

                        page.structure = _.object(_.pluck(itemList, 'id'), _.pluck(itemList, 'structure'));


                        self.values.allPages = ([].concat(self.values.allPages, itemList));
                        return page;
                    });


                    return _.filter(pages, function (page) {

                        if(page.type == 'dialog'){
                            return false;
                        }

                        if (cNav && !isConfigUpdated) {
                            //if it exists in the nav use it :)
                            return _.has(cNav, page.id);
                        }
                        else {
                            return  _.has(page, 'role') && _.has(page.role, role);
                        }
                    }, this);
                };

            if(isConfigUpdated){
                currentNav = null;
            }


            /**
             * Set up all dialogs that can be accessed from the course
             * This allows us to access the dialog help and such using hte same navigation calls as a page removing need for unwanted calls
             * @type {Array}
             */
            self.values.allDialogs =  _.filter(data.pages,function(page){
                   if(page.type == 'dialog'){
                       return true;
                   }
                   return false;
               });

            //filter all pages within the data
            data.pages = pageFilter(data.pages, currentNav);

            self.conditional.lockables = (_.object(_.pluck(data.pages,'id'), _.pluck(data.pages,'conditions')));
            self.conditional.lockables = _.omit(self.conditional.lockables, function(value, key, object) {return value === undefined;});


            /**
             * When the navigation is initialized the suspend data will be generated and submitted.
             * The first initialize call on the usspenddata will mean the data is saved and we can progres to the next step
             */
            self.once("initialized:suspendData", function () {
                if (!self.get('suspendData')) {
                    self.loadFrom(self.dataModel.getSuspendData(), self);
                    self.setSuspendDataIndices();
                    console.log("Suspend data was successfully initialized");
                }
                deferred.resolve(data);
            });



            self.set('nav', _.object(_.pluck(data.pages, 'id'), _.pluck(data.pages, 'structure')), {silent: true});

            //handles cases when the suspend data string doesnt update and the execuction fails
            self.trigger("change", self);

            self.values.allPages = ([].concat(self.values.allPages, data.pages));
            return deferred.promise();
        },

        /**
         * Update the suspend data
         *
         * The suspend data will be reloaded everytime there is an update to the values within the model
         *
         * @param updateType
         * @returns {*}
         */
        updateSuspendData: function (updateType) {

            var self = this,
                currentSuspendData = (self.get('suspendData') ),
                updatedModel = self.models.currentSuspendData.attributes,
                qued = function () {
                    var deferred = $.Deferred();

                    if (!window.quedSuspendDataUpdate) {
                        window.quedSuspendDataUpdate = true;
                        window.suspendDataReload = false;
                        self.suspendDataChanged()
                            .always(function () {
                                window.quedSuspendDataUpdate = false;
                                if (window.suspendDataReload) {
                                    window.suspendDataReload = false;
                                    qued();
                                }
                                deferred.resolve();
                                self.trigger("updated:suspendData", self,{updateType:updateType});
                            });
                    }
                    else {
                        window.suspendDataReload = true;
                    }

                    return deferred.promise();
                };


            /**
             * IF there is no suspend data then reload the suspend data from the LMS
             */
            if (currentSuspendData === null) {
                self.loadFrom(self.dataModel.getSuspendData(), self);
                currentSuspendData = (self.get('suspendData'));
                self.setSuspendDataIndices();
            }


            if (self.values.pageSuspendIdx == -1 || !_.has(currentSuspendData, self.values.pageSuspendIdx)) {
                console.warn("index not found/ Not updated");
            }
            else {
                currentSuspendData[self.values.pageSuspendIdx] = updatedModel;
                self.set('suspendData', currentSuspendData);



                if (updateType == 'completion') {
                    /**
                     * IF completion of the page has changed then we need to recalcualte completion of the course
                     */
                    self.recalculateCompletion()
                        .always(qued);
                }

                else if (updateType == 'scoring') {
                    /**
                     * IF score of the page has changed then we need to recalcualte score of the course
                     */
                    self.recalculateScoring()
                        .always(qued);
                }
                else if (updateType == 'scoringAndCompletion'){
                    self.recalculateScoring()
                        .always(function(){
                            console.log("Recalculating completion");
                            self.recalculateCompletion().always(qued);
                        });

                }

                else {

                    window.updateOtherData = true;
                    if (!window.quedSuspendDataUpdate) {
                        qued();
                    }
                }
            }
            return self;
        },

        /**
         * Re calculate completion of the course and the topics
         * @returns {*}
         */
        recalculateCompletion: function () {
            var
                self = this,
                deferred = $.Deferred(),
                nav = this.get('nav'),
                currentSuspendData = this.get('suspendData'),
                preferred = this.get('preferred'),
                allPages = this.values.allPages,
                completion = [],
                allTrackable = [],
                dataModel = self.dataModel,
                hashMap = {
                    'completion': _('completion').getKeyHash(),
                    'not_attempted': _('not attempted').getKeyHash(),
                    'incomplete': _('incomplete').getKeyHash(),
                    'completed': _('completed').getKeyHash(),
                    'passed': _('passed').getKeyHash(),
                    'failed': _('failed').getKeyHash()
                },

                /**
                 * The main recalculate function.
                 * STEP 1 : Find all trackable pages (page.track.completion == true )
                 * STEP 2 : For each of the above pages get the suspend data and the completion
                 * STEP 3 : If this page has sub items them recursively calculate the completion of this page
                 * STEP 4 : Based on the total number of not-attempted pages / incomplete / completed pages calcualte group / coruse completion
                 *          If there are incomplete pages + not attempted pages then the course / group status is incomplete
                 *          If all sub pages are completed / passed then the even bubbles up
                 */
                recalculate = function (pages, trackable , preventCommit) {
                    preventCommit  = preventCommit || false;
                    var trackablePages = _.filter(pages, function (page) {
                        var trackable = true,
                            cp  = self.get('currentPool');

                        trackable = trackable && (page.track && page.track.completion);

                        if(cp && page.track && page.track.pool){
                            trackable = trackable && ((page.track.pool+"") === (cp+""));
                        }

                        console.log(page.id +" ==> trackable ?? " + trackable);
                        return trackable;
                    }, this);
                    trackable = trackable || [];


                    _.map(trackablePages, function (page) {
                        var suspendKey = _.findKey(currentSuspendData, {id: page.id}),
                            it,
                            subTrackables,
                            pageCompletion,
                            counter;


                        if (!suspendKey) {
                            currentSuspendData = currentSuspendData || {};
                            currentSuspendData[page.id] = {id: page.id};
                            currentSuspendData[page.id][hashMap.completion] = hashMap.not_attempted;
                            suspendKey = _.findKey(currentSuspendData, {id: page.id});
                        }

                        it = currentSuspendData[suspendKey];
                        it.type = page.type;


                        if (!_.has(page, 'items') || page.items.length == 0) {
                            trackable.push(it);
                            return page;
                        }


                        subTrackables = (recalculate(page.items, []));
                        pageCompletion = _.pluck(subTrackables, hashMap.completion);

                        counter = _.countBy(pageCompletion, function (completion) {
                            if (!completion || completion == _('not attempted').getKeyHash()) {
                                return _('not attempted').getKeyHash();
                            }
                            return completion;
                        });


                        if(counter[hashMap.not_attempted] == trackablePages.length){
                            it[hashMap.completion] = hashMap.not_attempted;
                        }
                        else if (counter[hashMap.not_attempted] > 0 || counter[hashMap.incomplete] > 0 || counter[hashMap.failed] > 0) {
                            it[hashMap.completion] = hashMap.incomplete;
                        }
                        else {
                            var sendCompletedStatus=true;
                            if (self.courseConfig && self.courseConfig.completion && (self.courseConfig.completion.status == 'passed-failed-completed' || self.courseConfig.completion.status == 'passed-completed')) {
                                if (counter[hashMap.passed] > 0) {
                                    it[hashMap.completion] = hashMap.passed;
                                    sendCompletedStatus=false;
                                }
                                else {
                                    it[hashMap.completion] = hashMap.completed;
                                    sendCompletedStatus=false;
                                }
                            }
                            if(sendCompletedStatus){
                                it[hashMap.completion] = hashMap.completed;
                            }
                        }

                        trackable.push(it);
                        currentSuspendData[suspendKey][hashMap.completion] = it[hashMap.completion];

                        self.set('suspendData', currentSuspendData,{silent:true});
                        return page;
                    });

                    return trackable;
                };


            setTimeout(function () {
                allTrackable = recalculate(preferred.pages, allTrackable);

                /**
                 * Get completion of all topics
                 * @type {Array}
                 */
                var allCompletion = _.pluck(allTrackable, hashMap.completion),
                    allCounter = _.countBy(allCompletion, function (completion) {
                        if (!completion || completion == hashMap.not_attempted) {
                            return hashMap.not_attempted;
                        }
                        return completion;
                    });


                /**
                 * Set course status based on topic completion
                 */
                if (allCounter[hashMap.not_attempted] > 0 || allCounter[hashMap.incomplete] > 0 || allCounter[hashMap.failed] > 0) {
                    self.set('courseStatus',hashMap.incomplete,{silent:true});
                    console.log("Lesson status :incomplete");
                    dataModel.setLessonStatus('incomplete');
                    dataModel.commit();
                }
                else {

                    var sendCompletedStatus=true;

                    if (self.courseConfig && self.courseConfig.completion){
                        if(self.courseConfig.completion.status == 'passed-failed-completed' || self.courseConfig.completion.status == 'passed-completed' ||  (self.courseConfig.completion.status == 'completed-with-score')) {
                            if (allCounter[hashMap.passed] > 0) {
                                console.log("Lesson status :passed");
                                self.set('courseStatus', hashMap.passed, {silent: true});
                                dataModel.setLessonStatus('passed');
                                dataModel.commit();
                                sendCompletedStatus = false;
                            }
                            else if (allCounter[hashMap.failed] > 0) {
                                console.log("Lesson status :failed");
                                self.set('courseStatus', hashMap.failed, {silent: true});
                                dataModel.setLessonStatus('failed');
                                dataModel.commit();
                                sendCompletedStatus = false;
                            }
                        }
                    }

                    if(sendCompletedStatus){
                        console.log("Lesson status :completed");
                        self.set('courseStatus',hashMap.completed,{silent:true});
                        dataModel.setLessonStatus('completed');
                        dataModel.commit();
                    }
                }




                self.suspendDataChanged(true)
                    .always(function () {
                        deferred.resolve();
                    })
            }, window.pageLoadSpeed);

            return deferred.promise();
        },

        /**
         * @tpdp // complete ths
         * @returns {*}
         */
        recalculateScoring: function () {
            var
                self = this,
                deferred = $.Deferred(),
                nav = this.get('nav'),
                currentSuspendData = this.get('suspendData'),
                preferred = this.get('preferred'),
                allPages = this.values.allPages,
                scores = [],
                allScorable = [],
                dataModel = self.dataModel,
                hashMap = {
                    'score' : _('score').getKeyHash()
                },

                /**
                 * Recalculate course scoring based on page scores
                 */
                recalculate = function (pages, trackable) {
                    var trackablePages = _.filter(pages, function (page) {
                            var trackable = true,
                                cp  = self.get('currentPool');

                            trackable = trackable && (page.track && page.track.score && page.track.score===true) ;

                            if(cp && page.track && page.track.pool){
                                trackable = trackable && ((page.track.pool+"") === (cp+""));
                            }

                            return trackable;

                        }, this),
                        trackable = trackable || [],
                        score = 0,
                        allScores=[];




                    _.map(trackablePages, function (page) {
                        var suspendKey = _.findKey(currentSuspendData, {id: page.id}),
                            it,
                            subTrackables,
                            completion,
                            pageCompletion,
                            counter,
                            total=0;

                        if (!suspendKey) {
                            currentSuspendData = currentSuspendData || {};
                            currentSuspendData[page.id] = {id: page.id};
                            currentSuspendData[page.id][hashMap.completion] = hashMap.not_attempted;
                            suspendKey = _.findKey(currentSuspendData, {id: page.id});
                        }

                        it = currentSuspendData[suspendKey];
                        it.type = page.type;

                        if (!_.has(page, 'items') || page.items.length == 0) {
                            trackable.push(it);
                            return page;
                        }

                        subTrackables = (recalculate(page.items,[]));

                        allScores= _.pluck( subTrackables,hashMap.score);


                        total = _.reduce(allScores,function(memo,score){
                            if(!score){score = 0;}
                            else{
                                score = parseInt(score);
                                if(_.isNaN(score)){ score = 0;}
                                else if(score > 100){score = 100;}
                                else if(score < 0 ){score = 0;}
                            }

                            return memo+ (score || 0);
                        },0);


                        it[hashMap.score]=total;
                        trackable.push(it);



                        currentSuspendData[suspendKey][hashMap.score] = (it[hashMap.score]/allScores.length);
                        self.set('suspendData', currentSuspendData,{silent:true});
                        return page;
                    });

                    return trackable;
                };



            setTimeout(function () {
                allScorable = recalculate(preferred.pages, allScorable);
                scores = _.pluck(allScorable,hashMap.score);

                if(!scores || scores.length == 0){
                    //no update
                    deferred.resolve();
                }


                var total = _.reduce(scores,function(memo,score){
                    if(!score){score = 0;}
                    else{
                        score = parseInt(score);
                        if(_.isNaN(score)){ score = 0;}
                        else if(score > 100){score = 100;}
                        else if(score < 0 ){score = 0;}
                    }
                    return memo+ (score || 0);
                },0);

                self.set('primaryScore',(total/scores.length),{silent:true});

                self.suspendDataChanged(true)
                    .always(function () {deferred.resolve();})


                console.log("Save complete ");
            }, window.pageLoadSpeed);

            return deferred.promise();
        },


        /**
         * Set up the suspend data indices
         * @returns {boolean}
         */
        setSuspendDataIndices: function () {
            var self = this,
                currentSuspendData = self.get('suspendData') || {},
                suspend;

            if (!currentSuspendData || !self.values.pageNav || !self.values.pageConfig) {
                return false;
            }

            //find the suspend data
            self.values.pageSuspendIdx = _.findKey(currentSuspendData, function (x) {
                return x.id == self.values.pageConfig.id;
            });

            //get the previous suspend data
            self.values.previousSuspendData = suspend = (!self.values.pageSuspendIdx || self.values.pageSuspendIdx == -1) ? {} : currentSuspendData[self.values.pageSuspendIdx];
            //get the suspend default suspend value
            suspend = self.suspendDataDefaults(suspend, self.values.pageNav, self.values.pageConfig)
            self.models.currentSuspendData = new window.bazingaApp.models.pageData(suspend);

            self.models.currentSuspendData.on("change", function () {
                var changed = self.models.currentSuspendData.changed;

                if (_.has(changed, (_('completion').getKeyHash()))) {

                    self.updateSuspendData('completion');
                }
                else if (_.has(changed, (_('score').getKeyHash()))) {
                    self.updateSuspendData('scoring');
                }
                else {
                    self.updateSuspendData();
                }

            });

        },
        /**
         * Initialize the current screen
         * @returns {*}
         */
        setUpCurrentScreen: function () {
            var location = window.location.pathname,
                locationHref = (location.split("/")).pop(),
                self = this,
                config,
                role  = self.get('role') || 'default',
                page,
                currentScreen = self.get('currentScreen'),
                previousScreen = self.get('previousScreen');


            if (self.values.allPages.length == 0) {
                return false;
            }

            self.values.pageNav = page = _.first(_.where(self.values.allPages, {resource: locationHref}));

            if (self.values.pageNav) {


                //set up the navigation model.
                self.models.currentPageNav = new window.bazingaApp.models.pageNav(page);


                //get the config
                self.values.pageConfig = config = _.extend({ "count": true, "data": true, "completion": true, "score": false, "id": page.id}, page.track),

                    //find the index of the page navigation
                    self.values.pageNavIdx = _.findIndex(self.values.allPages, function (x) {
                        return x.id == page.id;
                    });

                self.setSuspendDataIndices();

                self.dataModel.setLessonLocation(self.values.pageNav.id);

                if(["dcom","dcon","dex","dh","dr"].indexOf(self.get('currentScreen')) == -1){
                    //reset the data
                    self.set('previousScreen', self.get('currentScreen'), {silent: true});//currentScreen:null,
                    self.set('currentScreen', self.values.pageNav.id, {silent: true});
                }

                self.values.isDialogScreen=false;
                return self.values.pageNav.resource;
            }
            else{
                self.values.dialogNav=_.first(_.where(self.values.allDialogs, {resource: locationHref}));
                self.values.isDialogScreen = true;

                _.map(self.values.allDialogs,function(dialog){
                    if(!dialog.role || !dialog.role[role] || !dialog.role[role].nav){return dialog;}

                    var nav = dialog.role[role].nav;
                    if(_.isEmpty(nav)){return dialog;}

                    _.each(nav,function(v,k){
                        if(v === 'currentScreen' || v === "<<<"){
                            nav[k] = self.values.dialogNav.id == 'dcon' ? previousScreen : currentScreen;
                        }
                    });

                    dialog.role[role].nav = nav;
                    return dialog;
                });
            }


        },

        /**
         * Initialize the page variables
         * If mark_course_complete_on_enter is true then the course will be automatically be marked as complete when this page is visited.
         * @returns {*}
         */
        pageInitializers: function () {
            var self = this,
                deferred = $.Deferred(),

                commitCourseStatus = function(){
                    var deferred = $.Deferred();
                    setTimeout(function () {
                        if (self.values && self.values.pageConfig && (self.values.pageConfig.mark_course_complete_on_enter)) {
                            self.dataModel.setLessonStatus('completed');
                            self.dataModel.commit("");
                        }
                        deferred.resolve();
                    }, window.pageLoadSpeed);
                    return deferred.promise();
                },
                commitPageStatus = function () {
                    var deferred = $.Deferred();

                    if (self.values && self.values.pageConfig && (self.values.pageConfig.completion && self.values.pageConfig.force_complete_on_enter)) {
                        self.models.currentSuspendData.setStatus('completed');
                    }
                    setTimeout(function () {deferred.resolve();}, window.pageLoadSpeed);
                    return deferred.promise();
                },
                commitCount = function () {
                    var deferred = $.Deferred();
                    if (self.values && self.values.pageConfig && self.values.pageConfig.count) {
                        self.models.currentSuspendData.incrementCount();
                    }
                    setTimeout(function () {deferred.resolve();}, window.pageLoadSpeed);
                    return deferred.promise();
                },
                initializePage = function(){
                    var deferred = $.Deferred();
                    if(self.values && self.values.pageConfig){
                        if((self.values.pageConfig.completion && self.values.pageConfig.force_complete_on_enter)){
                            self.models.currentSuspendData.setStatus('completed');
                        }

                        if(self.values.pageConfig.count){
                            self.models.currentSuspendData.incrementCount();
                        }

                        if(self.values.pageConfig.auto_reset_on_enter){
                            self.reattemptScreen(null,'default',true);
                        }

                        if((self.values.pageConfig.mark_course_complete_on_enter)){
                            self.dataModel.setLessonStatus('completed');
                        }

                        if(self.values.pageConfig.complete_course_on_score){
                            var primaryScore = self.get('primaryScore');
                            if(_.isNumber(primaryScore) && _.isNumber(self.values.pageConfig.complete_course_on_score) && (primaryScore >= self.values.pageConfig.complete_course_on_score)){
                                self.setCourseScoreFrom('primary');
                                self.models.currentSuspendData.setStatus('completed');
                                self.dataModel.setLessonStatus('completed');
                            }
                        }

                    }

                    setTimeout(function () {deferred.resolve();}, window.pageLoadSpeed);
                    return deferred.promise();
                };


                //orderly commit.
//                commitPageStatus().always(function(){
//                    commitCount().always(function(){
//                        commitCourseStatus().always(function(){
//                            self.suspendDataChanged(true).always(function(){deferred.resolve();})
//                        });
//                    })
//                });

            initializePage()
                .always(function(){
                    self.suspendDataChanged(true).always(function(){deferred.resolve();})
                });

            self.trigger("initialize:page", self);
            return deferred.promise();
        },

        getPageScore: function () {
            return this.models.currentSuspendData.getScore();
        },

        setMultiple : function(values){
            var deferred = $.Deferred(),
                self = this;
            setTimeout(function () {

                _.each(values,function(v){
                    if(v.key == 'score'){
                        self.dataModel.setScore(v.value);
                    }
                    else if(v.key == 'status'){
                        self.dataModel.setLessonStatus(v.value);
                    }
                });

                self.dataModel.commit("");
                deferred.resolve();
            }, window.pageLoadSpeed);

            return deferred.resolve();
        },

        setCourseScore : function(score){
            var deferred = $.Deferred(),
                self = this;
            setTimeout(function () {
                self.dataModel.setScore(score);
                self.dataModel.commit("");
                deferred.resolve();
            }, window.pageLoadSpeed);

            return deferred.promise();
        },
        setCourseStatus : function(status){
            var deferred = $.Deferred(),
                self=this;

            setTimeout(function () {
                self.dataModel.setLessonStatus(status);
                self.dataModel.commit("");
                deferred.resolve();
            }, window.pageLoadSpeed);

            return deferred.promise();
        },

        /**
         * Set up the course score from one of the primary / secondary / tertiary scores
         * @param scoreFrom
         */
        setCourseScoreFrom : function(scoreFrom){
            scoreFrom = scoreFrom || 'primary';
            var score,
                self=this,
                deferred= $.Deferred();


            switch(scoreFrom){
                case 'secondary':
                    score = this.get('secondaryScore') || -1;
                    break;
                case 'tertiary':
                    score = this.get('tertiaryScore') || -1;
                    break;
                case 'primary':
                default:

                    score = this.get('primaryScore') || -1;
                    break;
            }

            setTimeout(function () {
                if(score != -1){
                    self.dataModel.setScore(score);
                    self.dataModel.commit("");
                }
                
                deferred.resolve();
            }, window.pageLoadSpeed);

            return deferred.promise();
        },
        getPageCompletion: function () {
            var suspendData = this.currentSuspendData || this.models.currentSuspendData;
            if(suspendData && typeof suspendData.getCompletion === 'function'){
                return this.models.currentSuspendData.getCompletion();
            }
            return null;
        },

        getPageData: function () {
            return this.models.currentSuspendData.getData();
        },

        getPageCount: function () {
            if(this.models.currentSuspendData == null){
                return;
            }
            return this.models.currentSuspendData.getCount();
        },
        getAllPages: function () {
            return this.values.allPages;
        },

        getInfo: function (key) {
            return this.get(key);
        },

        setPrimaryScore : function(score){
            return this.set('primaryScore',score);
        },
        setCourseData : function(data){
            return this.set('courseData' ,data);
        },
        getCourseData : function(){
            return this.get('courseData');
        },
        getPrimaryScore : function(){
          return this.get('primaryScore');
        },
        setSecondaryScore : function(score){
            return this.set('secondaryScore',score);
        },
        getSecondaryScore : function(){
          return this.get('secondaryScore');
        },
        setTertiaryScore : function(score){
            return this.set('tertiaryScore',score);
        },
        getTertiaryScore : function(){
          return this.get('tertiaryScore');
        },

        getIsDialog : function(){
            return this.values.isDialogScreen;
        },

        getPreviousScreen: function () {
            return this.get('previousScreen');
        },
        getCurrentScreen: function () {
            return this.get('currentScreen');
        },

        getPageDataForPage: function () {
            return this.models.currentPageNav;

        },
        getConfigForScreen: function () {
            return this.values.pageConfig;
        },
        getNavForScreen: function (id) {
            if (!id) {return this.values.pageNav;}
            return (_.first(_.where(this.values.allPages, {id:(""+id+"")})));
        },
        getNavForDialog : function(id){
            //get the nav for the right screen .
            return (_.first(_.where(this.values.allDialogs, {id:id})));
        },
        getNavForScreenByRole: function (role,id) {
            role = role || 'default';

            var nav;
            if(!id){
                nav = this.values.pageNav;
            }
            else{
                nav = this.getNavForScreen(id);
            }
            return (nav && nav.role && nav.role[role]) ? nav.role[role].nav : null;

        },

        getSuspendData  : function(){
            return this.get('suspendData');
        },
        getRole: function () {
            return this.get('role');
        },

        getNav: function () {
            return this.get('nav');
        },

        getDecision: function (decisionPoint, role,id) {
            var self = this,
                navForRole = _.memoize(self.getNavForScreenByRole),
                nav,
                navForDialog,
                role = self.get('role')||'default';

            if(this.getIsDialog()){
                navForDialog =  self.values.dialogNav;
                if(navForDialog && navForDialog.role && navForDialog.role[role]){
                    nav = navForDialog.role[role].nav;
                    if(!nav){return false;}
                    return nav[decisionPoint] || 'home';
                }
            }
            else{
                nav = navForRole(role,id);

                if(nav){
                    return nav[decisionPoint];
                }
            }

            return null;
        },

        getAllDialogs : function(){
            return this.values.allDialogs;
        },
        getDialog : function(dialogname){
            var dialogs = this.values.allDialogs;

            return _.first(_.where(dialogs,{id:(_(dialogname).getKeyHash())}));
        },

        getLockables : function(){
            return this.conditional.lockables;
        },
        getCrumbs  : function(id){
            var self = this,
                nav,
                crumbs = [],
                ret;

            if(this.getIsDialog()){
                return self.values.dialogNav['title'];
            }
            nav =  self.getNavForScreen(id);
            crumbs = nav.crumbs;


            if(!_.isEmpty(crumbs)){
                ret = [];
                _.each(crumbs,function(v){
                    var n = self.getNavForScreen(v);
                    ret.push(n);
                });
                return ret;
            }

            return crumbs;
        },

        setPageCompletion: function (completion) {
            if(!this.models || !this.models.currentSuspendData){
                console.warn("Cannot locate model to set page completion");
                return this;
            }
            this.models.currentSuspendData.setCompletion(completion);
            return this;
        },

        reattemptScreen : function(id,role,preventCommit){
            var self =this,
                nav,
                reset,
                resetScores,
                resetStatus,
                resetData,
                deferred = $.Deferred(),
                currentSuspendData,
                hashed  = {
                  'completion' : _("completion").getKeyHash(),
                  'completed' : _("completed").getKeyHash(),
                  'passed' : _("passed").getKeyHash(),
                  'not_attempted' : _("not attempted").getKeyHash(),
                  'score' :  _("score").getKeyHash(),
                  'data' :  _("data").getKeyHash()
                };

           role = role || 'default';

            if(this.values.isDialogScreen){
                nav = this.values.dialogNav;
            }
            else if(!id){
                nav = this.values.pageNav;
            }
            else{
                nav = this.getNavForScreen(id);
            }
            if(!nav || !_.has(nav,'role') || !nav.role[role]){
                return false;
            }
            nav = nav.role[role];

            if(!_.has(nav,'reset')){return false;}
            reset = nav.reset;

            resetScores = _.has(reset,'score');
            resetStatus = _.has(reset,'status');
            resetData = _.has(reset,'data');


            if(_.has(reset,'primaryScore')){
                self.set('primaryScore',null);
            }

            currentSuspendData = self.get('suspendData');

            _.map(currentSuspendData, function(v){
                if(resetScores && reset.score.indexOf(v.id) > -1){
                    v[hashed.score] = 0;
                }

                if(resetStatus && reset.status.indexOf(v.id)>-1){
                    v[hashed.completion]=hashed.not_attempted;
                }

                if(resetData && reset.data.indexOf(v.id)>-1){
                    v[hashed.data]=null;
                }

                return v;
            });

            if(!preventCommit){
                self.once("updated:suspendData",function(){
                   deferred.resolve();
                });

                self.updateSuspendData('scoringAndCompletion');
            }
            else{
                setTimeout(function(){
                    deferred.resolve();
                }, window.pageLoadSpeed);
            }

            return deferred.promise();
        },
        setPageData: function (data) {
            this.models.currentSuspendData.setData(data);
            return this;
        },
        setInfo: function (key, value, options) {
            this.set(key, value);
            return this;
        },
        setCurrentPool: function (poolId) {
            this.set('currentPool', poolId);
            return this;
        },
        getCurrentPool : function(){
           return this.get('currentPool');
        },
        setPageScore: function (score) {
            this.models.currentSuspendData.setScore(score);

            return this;
        },

        suspendDataDefaults: function (suspend, page, config) {
            suspend[_('id').getKeyHash()] = page.id;


            if (config.count) {
                suspend[_('count').getKeyHash()] = suspend[_('count').getKeyHash()] || 0;
            }
            if (config.data) {
                suspend[_('data').getKeyHash()] = suspend[_('data').getKeyHash()] || '';
            }
            //block track
            if (_.has(page, 'block-track') || !_.has(page, 'track')) {
                //completion
                if (config.completion) {
                    suspend[_('completion').getKeyHash()] = _(suspend[_('completion').getKeyHash()] || 'not attempted').getKeyHash();
                }
                //score
                if (config.score) {
                    suspend[_('score').getKeyHash()] = suspend[_('score').getKeyHash()] || '0';
                }
            }
            return suspend;
        },
        /**
         * Create a suspend data
         */
        retrieveSuspendData: function () {
            var attributes = this.attributes,
                self = this,
                hashedPage = _('page').getKeyHash(),
                suspendData = (function () {
                    var sus = {};
                    sus[hashedPage] = {};
                    return sus;
                }()),
                currentSuspendData = self.get('suspendData'),
                getData = function (pages) {
                    _.map(pages, function (page) {

                        var config = _.extend({ "count": true, "data": true, "completion": true, "score": false}, page.track),
                            suspend = (function () {
                                var data = _.filter(currentSuspendData, function (it) {
                                    return it.id == page.id;
                                });
                                if (data.length == 0) {
                                    return {}
                                }
                                return _.first(data);
                            }());


                        suspend = self.suspendDataDefaults((suspend || {}), page, config);

                        if(_.has(suspend,'type')){delete suspend.type;}


                        if (!_.has(page, 'items')) {
                            suspendData[hashedPage][page.id] = suspend;
                        }
                        else {
                            getData(page.items);
                        }
                    });
                };

            _.map(attributes, function (v, k) {
                var hashedKey = _(k).getKeyHash();

                if (k !== 'preferred') {
                    //Do nothing
                    if (k == 'suspendData') {
                        return true;
                    }
                    suspendData[hashedKey] = v;
                    return true;
                }
                getData(v.pages);
                return true;
            });


            return (function () {
                var stringified = _(suspendData).crunchSuspendData();
                return stringified;
            }());
        },


        setDataModel: function (dataModel) {
            this.dataModel = dataModel;
            if (typeof dataModel.setNavModel == 'function') {
                dataModel.setNavModel(this);
            }

            this.initializeDataModelListeners();
            return this;
        },
        setCourseConfig: function (courseConfig) {
            this.courseConfig = courseConfig;
            return this;
        },
        getCourseConfig: function(){
            return this.courseConfig;
        },
        initializeDataModelListeners: function () {

        }
    });

})(window.$ || window.JQuery);
(function($) {
	"use strict";
	$(function() {
		/**
         * control the visibility of global navigation bar
         */
		var conbinationMenu = $('#js-dropdown-mobile'),
		    mainMenu = $('#js-dropdown-mobile > ul:first-child'),
		    subMenu = $('#js-dropdown');
		function setVisibility() {
			if ($(window).width() <= 768) {//mobile
				conbinationMenu.addClass('collapse');//in mobile view, conbinationMenu should be hidden
				if (subMenu.hasClass('in')) {//if submenu has already opened in largeScreen
					conbinationMenu.addClass('collapse in');
				}
			} else {//large screen
				conbinationMenu.removeClass('collapse in');//in large screen, conbinationMenu should be shown
				if (conbinationMenu.hasClass('in')) {//conbinationMenu has already opened in mobile
					subMenu.addClass('collapse in');
				}
			}
		}

		//detect on page load
		setVisibility();
		//detect on window resize
		$(window).on('resize', function() {
			setVisibility();
		});
		//ready for show, remove all hidden classes
		$('.navbar').on('show.bs.collapse', function() {
			if ($(window).width() <= 768) {
				$('#js-dropdown-mobile > ul', $(this)).addClass('in');
			}
		});
		//ready for hidden, remove all display classes
		$('.navbar').on('hidden.bs.collapse', function(e) {
			if ($(window).width() <= 768) {
				if(e.target.id == 'js-dropdown-mobile'){
					subMenu.removeClass('in');
					mainMenu.removeClass('in');
				}
			}else{
				conbinationMenu.removeClass('in');
			}
		});
	});
})(window.$ || window.JQuery); 
(function($){
    "use strict";

    /**
     * @todo: convert to a template . So that the preloader gets compiled
     * @type {*}
     */
    window.bazingaApp.views.preloaderView = Backbone.View.extend({
        el:$("[data-preloader='1']"),
        selector : {
            'icon':"[data-icon]",
            'label':"[data-label]",
            'percentage':"[data-percentage]"
        },

        show : {
            'label':true,
            "percentage":true,
            "icon":true,
            "symbol" : "%"
        },

        shown : true,
        label : "Loading..",
        percentage : 0,
        delay : 15,


        initialize:function(){
            this.$el.find(this.selector.label).text(this.label);
            this.$el.find(this.selector.percentage).text(this.percentage);
            this.resetVisibleElements();
            _.bindAll(this, 'setPercentage', 'incrementTo');
        },
        setLabel : function(label){
            this.label = label;
            this.$el.find(this.selector.label).text(label);
            return this;
        },
        setPercentage : function(percentage){
            this.percentage = percentage;
            this.$el.find(this.selector.percentage).text(percentage + this.show.symbol);
            if(percentage == 100){
                this.shown = false;
                this.$el.hide();
            }
            else{
                if(!this.shown){
                    this.$el.show();
                    this.shown = true;
                }
            }
            return this;
        },

        incrementTo : function(incrementTo,delay){
            var current = this.percentage,
                self = this,
                interval,
                deferred = $.Deferred();
                delay = delay || self.delay || 30;

                interval = setInterval(function(){
                    if(current > 100){
                        current = 100;
                    }
                    else if(current > incrementTo){
                        current = incrementTo;
                    }
                    if(current == 100 || current == incrementTo){
                        clearInterval(interval);
                        deferred.resolve();
                        return false;
                    }
                    current +=1;
                    self.setPercentage(current);
                },delay);

                return deferred.promise();
        },
        resetVisibleElements: function(){
            if(this.show.icon){
                this.$el.find(this.selector.icon).show();
            }
            else{
                this.$el.find(this.selector.icon).hide();
            }

            if(this.show.percentage){
                this.$el.find(this.selector.percentage).show();
            }
            else{
                this.$el.find(this.selector.percentage).hide();
            }

            if(this.show.label){
                this.$el.find(this.selector.label).show();
            }
            else{
                this.$el.find(this.selector.label).hide();
            }

            return this;
        },
        setShow : function(show){
            _.extend(this.show,show );
            return this.resetVisibleElements();
        },

        enable : function(){
            this.$el.show();
            return this;
        },
        disable : function(){
            this.$el.hide();
            return this;
        }

    });
})(window.$||window.JQuery);
(function($){
    "use strict";

    /**
     * Backstetch view for js . TO use this file do the following
     * In appview initialize the backstetch view
     ```
     window.bazingaApp.views.components.backstretch = new window.bazingaApp.views.backstretchView();
     ```
     * Use data tags to initialize the backstetch view. There needs to be two data tags
     ```
     <div class="background" data-backstretch=1 data-backstretch-img='../img/splash_background.png'></div>
     ```
     * @type {*}
     */
    window.bazingaApp.views.backstretchView = Backbone.View.extend({
        el: "[data-baz-backstretch='1']",

        initialize:function(){
            this.componentType='backstretch';
            var self=this;
            self.resize();
            if(this.$el.length == 0){return false;}

            this.$el.backstretch(this.$el.data('bazBackstretchImg'));
            window.onresize = self.resize.bind(self);
        },
        resize : function(){
            if ($(window).width() <= 766) {
                this.$el.css({'position': 'relative'});
            } else {
                this.$el.css({'position': 'fixed'});
            }
            if($(window).height() <= 620) {
            	$('.splash .last').css('bottom','auto');
            } else {
            	$('.splash .last').css('bottom',20);
            }
        }
    });

})(window.$||window.JQuery);
(function ($) {
    "use strict";
    window.bazingaApp.views.abstractComponentView = Backbone.View.extend({
        completed: 'not-attempted',
        componentType:'abstract',
        enabled : true,

        getComponentType : function(){return this.componentType;},
        /**
         * Make sure this is included in the component js
         */
        isEnabled : function(){return this.enabled;},
        /**
         * Make sure this is included in the component js
         */
        getIsCompleted: function () {
            return this.completed == 'completed' || this.completed === true || this.completed === 1;
        },

        init : function(){

        }

    });
})(window.$ || window.JQuery);
(function ($) {
    "use strict";

    /**
     * show and hide popup.
     * 4 types of popup "popup", "dialog", "tooltip" and "overlay"
     *
     * Mandatory Attributes:
     * id (use to address the popup, for example: set data-reveal-object="#pop1" for button, the id of popup should be "pop1")
     * data-reveal-type (the way to render the popup, for example: "popup","dialog")
     * data-reveal-object (the related popup id)
     * 
     * Optional:
     * class="js-popup-focus" (if set to one of the html element inside in popup, the element will be focused after popup displayed.)
     * 					(if not set, will focus the popup self)
     * useage:
     * 
     * html structure for type: "popup":
     * popup:
     * <div id="pop1" data-reveal-target="popbox">...</div>
     * button:
     * <button data-reveal-type="popup" data-reveal-object="#pop1">...</button>
     *
     * html structure for type: "tooltip":
     * popup:
     * <aside id="pop2" data-reveal-target="tooltip">
     * 		...
     *  <a href="#" class="tip-close" tabindex="204" data-reveal-type="pop-close" data-reveal-object="#pop7" >
     * 		<span class="icon-close"></span>
     * 	</a>
     * </aside>
     * button:
     * <button data-reveal-type="tooltip" data-reveal-object="#pop2">...</button>
     * 
     * html structure for type: "overlay":
     * popup:
     * <section id="overlay" class="overlay" data-reveal-target="overlay">
	 *		<div class="container-fluid">
	 *			<div class="row">
	 *				<div class="col-sm-8 col-sm-offset-2">
	 *					<div class="js-overlay-content">
	 *					</div>
	 *				</div>
	 *			</div>
	 *		</div>
	 *	</section>
	 * popup data:
	 * <div id="overlay-data1" class="js-overlay-data">
	 * ...
	 * <a class="button-close" href="#" tabindex="399" data-reveal-target="pop-close" data-reveal-object="#overlay">
     *   <span class="icon-close"></span>
     * </a>
	 * </div>
     * button:
     * <a href="#" data-reveal-object="#overlay" data-reveal-data="#overlay-data1" data-reveal-type="overlay"></a>
     * 
     * html structure for type: "dialog": (use bootstrap dialog)
     * popup:
     * <div class="modal fade" id="popup-dialog" role="dialog" data-backdrop="static" data-keyboard="false" data-reveal-target="pop-dialog">
     *      <div class="modal-dialog" tabindex="300">
     *        <div class="modal-content">
     *          <div class="modal-header">
     *            <button type="button" class="close" data-dismiss="modal" aria-label="Close" tabindex="302"><span aria-hidden="true">&times;</span></button>
     *            <h4 class="modal-title"></h4>
     *          </div>
     *         <div class="modal-body" tabindex="301"></div>
     *       </div>
     *     </div>
     * </div>
     * button:	
     * <button data-reveal-type="dialog" data-reveal-data="#pop2" data-reveal-object="#popup-dialog" data-toggle="modal" data-target="#popup-dialog"></button>
     *
     *
     */
    window.bazingaApp.views.popupView = window.bazingaApp.views.abstractComponentView.extend({
        el: $('[data-reveal-type]:not(.js-prevent-interaction)'),
        events: {
            "click": "dispatch",
            "touchend": "dispatch",
            "keydown":"onKeyPress"
        },
        clickable: true,
        selector: {
        	container: '[data-reveal-type]',
        	popup: '[data-reveal-target="popbox"]',
        	tooltip: '[data-reveal-target="tooltip"]',
        	dialog: '[data-reveal-target="pop-dialog"]',
        	overlay: '[data-reveal-target="overlay"]',
        	close: '[data-reveal-target="pop-close"]'
        },
        initialize: function () {
            _.bindAll(this, 'dispatch', 'show', 'hide', 'appendContent', 'dialog', 'recalculate','doFocus','applyAccessibility','delegateCloseEvent','onKeyPress');
            this.delegateEvents();
            this.componentType='popup';
            if(this.$el.length == 0){
                this.enabled = false;
                this.completed= 'completed';
            }
            else{
                this.completed = "incomplete";
            }


            this.delegateCloseEvent();
            this.dialog();
            this.recalculate();
        },
        onKeyPress : function(e){
            if(e.keyCode==13){
                this.dispatch(e);
            }
        },
        recalculate: function () {
            if(this.getIsCompleted()) {
                return this;
            }

            var self = this,
                completed = self.completed = self.completed || false,
                totalCount = $(self.selector.container).not(':hidden').length,
                incompleteCount = $(self.selector.container).not('.is-visited').length;

            if( incompleteCount == 0){
                self.completed = 'completed';
                self.trigger('completed:reveal',self,{total : totalCount});
            }
            else if(incompleteCount !== totalCount){
                self.completed = 'incomplete';
            }

            return self;
        },
        dispatch: function (e) {
            var content,
            	target = e.target,
                self=this,
                $target = $(target).data('revealObject')?$(target):$(target).closest('[data-reveal-object]'),
                revealData = $target.data('revealData'),
                revealObj = $target.data('revealObject'),
                revealType = $target.data('revealType');
               
            if($target.hasClass('is-disabled')){return;}//example: locker for simple senario homepage
            
            self.type="";
            e.preventDefault();
            //a flag to control the click behavior will only be enabled after last animation
            if (!self.clickable) {return;}
            //selecting the current target
           	this.$el.removeClass('is-selected');
            //complete the target
            $target.addClass('is-selected')
                .addClass('is-visited');


            (function(){
                var r=  ($target.data('revealReflect'));

                r= ((r || "")+"").split(",");
                if(r.length == 0){return null;}

                _.each(r,function(v,k){
                    $('[data-reveal-id="'+v+'"]').addClass('is-visited');
                });
            })();


            switch (revealType) {
                case 'popup':
                	self.clickable = false;
                    //hide all others, keep only one opens
                    $(self.selector.popup).not(revealObj).hide();
                    this.show(revealObj);
                    break;
                case 'tooltip':
                	$(self.selector.tooltip).not(revealObj).hide();
                	this.show(revealObj);
                case 'dialog':
                	content = $(this.selector.dialog+':eq(0) .modal-body');//dialog content
                    this.appendContent(content,revealData);
                    break;
                case 'overlay':
                	self.type="overlay";
                	content = $(this.selector.overlay+' .js-overlay-content');//overlay content
                	self.appendContent(content,revealData);
                	self.show(revealObj,revealType);
                	break;
            }
            self.recalculate();
        },
        show: function (el) {
        	var self = this;
            $(el).fadeIn(function(){
            	setTimeout(function(){
            		self.clickable = true;
            	},100);

                self.doFocus($(this),true);
                if(["overlay"].indexOf(($(self.el).data('revealType')))  > -1){
                	self.applyAccessibility($(this),true);
                }
            });
        },
        hide: function (el) {
        	var self = this;
            $(el).fadeOut(function(){
            	var revealId = $(this).attr('data-reveal-id');

                self.doFocus(revealId ? $('[data-reveal-data=#' + revealId + ']') : $('[data-reveal-object="#' + $(this).attr('id') + '"]:not(' + self.selector.close + ')'), false);
                if(["overlay"].indexOf(($(self.el).data('revealType')))  > -1) {
                    self.applyAccessibility(revealId ? $('[data-reveal-data=#' + revealId + ']') : $('[data-reveal-object="#' + $(this).attr('id') + '"]:not(' + self.selector.close + ')'), false);
                }
            });
        },
        dialog: function () {
        	var self = this;
            self.doFocus($('.modal-dialog',$(this)));
        	$(this.selector.dialog+':eq(0)').on('shown.bs.modal',function(){
        		self.applyAccessibility($('.modal-dialog',$(this)));
        	});
        },
        appendContent: function (content,el) {
        	var self = this,
                $el = $(el);
        	
        	content.closest(self.selector.overlay).attr('data-reveal-id',$(el).attr('id'));
        	if($(el,content).length == 0 ){
                $el.appendTo(content);
                self.trigger("data:popup",self,{$tgt:content});
        	}
        	$('.js-overlay-data').hide();
        	$(el).show();
        },
        doFocus: function(el,isOpen){

            var focusEl = $('[class*=-focus]',el),
                tabEls = $(':tabbable,[tabindex]'), //converting to tabbable element a button input 

                self = this;
            el.removeClass('is-selected');
            //focus the target for accessibility
            if($(window).width()>768){
                if(focusEl.length > 0){
                    focusEl.focus();
                }else {
                    el.focus();
                }
            }

        },
        applyAccessibility: function (el,isOpen){
        	var focusEl = $('[class*=-focus]',el),
                tabEls = $(':tabbable,[tabindex]'), //converting to tabbable element a button input 

        		self = this;
        	el.removeClass('is-selected');
        	//focus the target for accessibility
        	if($(window).width()>768){
        		if(focusEl.length > 0){
	        		focusEl.focus();
	        	}else {
	        		el.focus();
	        	}
        	}
			
			//@since 1.0 make header and main content not arrow-able
			if(isOpen){
				$(".navbar").attr('tabindex','-1').attr('aria-hidden','true');
				$(".main").attr('tabindex','-1').attr('aria-hidden','true');
			}else{
				$(".navbar").removeAttr('tabindex').removeAttr('aria-hidden');
				$(".main").removeAttr('tabindex').removeAttr('aria-hidden');	
			}
			
            //Added tabbable elements 
        	//restrict tabindex in overlay      
        	tabEls.each(function(){
        		if(self.type=="overlay"&&$(this).closest(".overlay,.tip,body[class*=dialog-]").length==0){
        			var tabindex = $(this).attr('tabindex');
        			if(tabindex){
                        if(isOpen){
                            if(tabindex == 0){
                                $(this).attr('tabindex','-9827').attr('aria-hidden','true'); //if a tab index is 0 set it to a random number
                            }
                            else{
                                $(this).attr('tabindex','-'+tabindex).attr('aria-hidden','true');
                            }
                           
                        }else{
                            if(tabindex == "-9827"){
                                $(this).attr('tabindex','0').removeAttr('aria-hidden'); 
                            }
                            else if(tabindex == "-9828"){
                                $(this).removeAttr('tabindex').removeAttr('aria-hidden');
                            }
                            else{
                                $(this).attr('tabindex',tabindex.substring(1)).removeAttr('aria-hidden'); 
                            }
                            
                        }
                    }
                    else{
                        //for a tags / btns etc
                        if(isOpen){
                            $(this).attr('tabindex','-9828').attr('aria-hidden','true'); //another random number because its easy and i m hungry @sundar
                        }
                    }
                    
        		}
        	});
        },
        delegateCloseEvent: function(){
        	var self = this;
        	$('body').on('click',self.selector.close,function(e){
        		e.preventDefault();
        		self.hide($($(this).data('revealObject')));
        	});
        }
    });
})(window.$ || window.JQuery);
(function($) {
    "use strict";
    window.bazingaApp.views.checkboxManager = window.bazingaApp.views.abstractComponentView.extend({
        checkboxes: [],
        cbIdentifier: '[data-cbid]',
        showNeutralFeedback: false,
        autoComplete: false,
        cbType: 'single',
        trackable: true,
        partials: {
            type: 'of',
            tgt: null
        },
        correct: [],
        totalWeightage: 0,
        state: "not-attempted",
        getState: function() {
            return this.state;
        },
        getIsTrackable: function() {
            return this.trackable;
        },
        initialize: function() {
            _.bindAll(this, 'load', 'randomize', 'loadCheckboxes', 'recalculate', 'show', 'hide');
        },
        selectOrder: [],
        definedOrder: [],
        feedbacks: {
            show: null,
            hide: null
        },
        animations: 'fade',

        recalculate: function(e) {
            if (e && e.target && ($(e.target).hasClass('is-disabled'))) {
                e.stopPropagation();
                return false;
            }



            var self = this,
                correct = self.correct,
                neutral = self.neutral,
                allCheckboxes = self.$el.find(self.cbIdentifier),
                answers = {
                    "correct": [],
                    "wrong": []
                },
                score = 0,
                score_max = self.totalWeightage,
                allIds = [],
                i = 0,
                fb = 'naf';

            if (self.resetButton) {
                self.resetButton.addClass('is-disabled').attr("aria-disabled",true);
            }

            if (!self.autoComplete) {
                self.submitButton.addClass('is-disabled').attr("aria-disabled",true);
            }

            allCheckboxes.addClass('is-disabled').off('click');

            if (self.cbType == 'order') {
                _.each(self.definedOrder, function(v) {
                    var ix = self.selectOrder[i++];
                    allIds.push(v.id);

                    if (v.id == ix) {

                        answers.correct.push(ix);
                    } else {
                        answers.wrong.push(ix);
                    }
                });
            } else {
                _.each(allCheckboxes, function(cb) {
                    var $cb = $(cb),
                        id = "" + $cb.data('cbid'),
						assistText = (function(){
                            var assistText = $cb.find(".a11y-assist-text");
                            if(assistText.length > 0){
                                return assistText;     
                            }
                            $cb.append("<div class='a11y-assist-text'></div>")                            
                            assistText = $cb.find(".a11y-assist-text");
                            return assistText; 
                        }()),
                        isCorrect = (_.indexOf(correct, id) > -1),
                        isChecked = $cb.hasClass('is-selected'),
                        showIfCorrect = $cb.data('showIfCorrect'),
                        showIfWrong = $cb.data('showIfWrong');

                    allIds.push(id);
                    $cb.addClass(isCorrect ? 'is-correct' : 'is-wrong');
					assistText.text(isCorrect ? 'Correct answer' : 'Incorrect answer'); //Needed for IE 

                    if ((isCorrect && isChecked) || (!isCorrect && !isChecked)) {
                        answers.correct.push(id);
                        if(showIfCorrect){
                            self.show($(showIfCorrect),null,100);
                        }
                        $cb.addClass('is-answered-correct');
                        score += parseInt($(cb).data('weight') || 1);
                    } else {
                        $cb.addClass('is-answered-wrong');
                        if(showIfWrong){
                            self.show($(showIfWrong),null,100);
                        }
                        answers.wrong.push(id);
                    }
                });
            }


            var ret = {
                target: self,
                answers: answers,
                allIds: allIds,
                score: score,
                score_max: score_max,
                percentage: ((score / score_max) * 100)
            };


            if (allIds.length == answers.correct.length) {
                ret.state = self.state = "passed";
            } else {
                //trigger neutral
                if (self.partials.tgt) {
                    if (self.partials.type == 'of') {
                        if (answers.correct.length >= self.partials.tgt[0]) {
                            ret.state = 'partial';
                        } else {
                            ret.state = 'failed';
                        }
                    } else if (self.partials.type == 'in') {
                        var overlap = _.intersection(self.partials.tgt, answers.correct);
                        if (!_.isEmpty(overlap) && (overlap.length == self.partials.tgt.length)) {
                            ret.state = 'partial';
                        } else {
                            ret.state = 'failed';
                        }
                    } else {
                        if (ret.percentage >= parseInt((self.partials.tgt[0] || 50))) {
                            ret.state = 'partial';
                        } else {
                            ret.state = 'failed';
                        }
                    }
                } else {
                    ret.state = 'failed';
                }
            }
            fb = '[data-caf]';
            if (self.showNeutralFeedback) {
                fb = '[data-naf]';
            } else {
                if (ret.state == 'passed') {
                    fb = '[data-caf]';
                } else if (ret.state == 'failed') {
                    fb = '[data-waf]';
                } else {
                    fb = '[data-pcaf]';
                }
            }

            if (self.feedbacks.show) {
                self.show(self.feedbacks.show.find(fb, null, 100));



                var showFeedback = function(callback) {
                        self.show(self.feedbacks.show, callback);
                    },
                    hideFeedback = function(callback) {
                        if (self.feedbacks.length == 0 || !_.has(self.feedbacks, "hide") || self.feedbacks.hide.length == 0) {
                            callback();
                        } else {
                            self.hide(self.feedbacks.hide, callback);
                        }
                    };


                if (self.animateOrder == 'hideShow') {

                    hideFeedback(function() {
                        showFeedback(function() {
                        	var el = self.feedbacks.show,
                        		navBar = $('.navbar');
                            self.trigger("completed:checkbox", ret);
                            //nice to have: scroll to feedback section on mobile view
							if($(window).width()<767&&el.hasClass("js-scrollTop")){
								$("html,body").animate({scrollTop:el.offset().top-navBar.outerHeight()},500,function(){
									(el.find(fb).find('[tabindex]:first').focus());
								});
							}
							
							////@since 1.0 add focus even on desktop for accessibility
							(el.find(fb).find('[tabindex]:first').focus());
							
                        });
                    });
                } else {
                    showFeedback(function() {
                        hideFeedback(function() {
                            self.trigger("completed:checkbox", ret);
                            (self.feedbacks.show.find(fb).find('[tabindex]:first').focus());
                        });
                    });
                }
            } else {
                setTimeout(function() {
                    self.trigger("completed:checkbox", ret)
                }, 20);
                console.warn("No feedbacks located. SO event was triggered");
            }

            self.completed = 'completed';
        },

        show: function(element, callback, delay) {
            var self = this,
                promise,
                classOverride = element.data('animationHide') || self.animations;

            delay = delay || 300;


            if (typeof callback !== 'function') {
                callback = function() {}
            }

            switch (classOverride) {
                case "fade":
                case "fadeinout":
                    promise = element.fadeIn(delay, callback);
                    //promise = element.fadeIn(delay, callback).css("display","inline-block");
                    break;

                case "slideUp":
                case "slideUpDown":
                    promise = element.slideUp(delay, callback);
                    break;

                case "slideDown":
                case "slideDownUp":
                    promise = element.slideDown(delay, callback);
                    break;

                default:
                    promise = element.show(delay, callback);
                    break;
            }

            return promise;
        },
        hide: function(element, callback, delay) {
            var self = this,
                promise,
                classOverride = element.data('animationHide') || self.animations;

            delay = delay || 600;

            if (typeof callback !== 'function') {
                callback = function() {}
            }


            switch (classOverride) {
                case "fade":
                case "fadeinout":
                    promise = element.fadeOut(delay, callback);
                    break;

                case "slideUp":
                case "slideDownUp":
                    promise = element.slideUp(delay, callback);
                    break;

                case "slideDown":
                case "slideUpDown":
                    promise = element.slideDown(delay, callback);
                    break;

                default:
                    promise = element.hide(delay, callback);
                    break;
            }

            return promise;
        },
        loadCheckboxes: function(reload) {
            reload = reload === false ? false : true;

            if (reload) {
                this.checkboxes = [];
            };


            var self = this,
                i = 1,
                checkboxes = this.$el.find(this.cbIdentifier),
                
                
                clickHandler = function(e) {
                    if (e && e.target && ($(e.target).hasClass('is-disabled'))) {
                        e.stopPropagation();
                        return false;
                    }

                    var tgt = e.target,
                        $target = $(tgt),
                        data,
                        id,
                        totalCheckboxes = self.checkboxes.length,
                        notSelected;



                    data = $target.data();
                    id = data.cbid; 

                    if (_.has(data, 'showSelector') && data.showSelector) {
                        self.show($(data.showSelector));
                    }
                    if (_.has(data, 'hideSelector') && data.hideSelector) {
                        self.hide($(data.hideSelector));
                    }


                    if ($target.hasClass('is-selected')) {
                        $target.removeClass('is-selected').attr("aria-checked",false);

                        //set up order
                        if (self.cbType == 'order') {
                            //remove the id from the list if it is not selected

                            var currentIx = _.indexOf(self.selectOrder, id);
                            if (currentIx > -1) {
                                //remove the current select order
                                self.selectOrder = _.reject(self.selectOrder, function(num) {
                                    return (("" + num) == ("" + id));
                                });
                                $target.find('[data-cb-selected-order]').remove();


                                //redo other select orders
                                _.each(self.selectOrder, function(so) {
                                    var soOb = $('[data-cbid="' + so + '"]').find('[data-cb-selected-order]'),
                                        selectedOrder = (soOb.data('cbSelectedOrder'));

                                    selectedOrder = parseInt(selectedOrder);
                                    if (!_.isNaN(selectedOrder) && selectedOrder > currentIx) {
                                        selectedOrder = (selectedOrder - 1);

                                        soOb
                                            .data('cbSelectedOrder', ("" + selectedOrder))
                                            .html(("" + selectedOrder));

                                    }
                                });
                            }



                        }
                    } else {
                        if (self.cbType == 'single') {
                            self.$el.find(self.cbIdentifier).removeClass('is-selected').attr("aria-checked",false);
                            
                        }
                        $target.addClass('is-selected').attr("aria-checked",true);


                        //set up order
                        if (self.cbType == 'order') {
                            if (_.indexOf(self.selectOrder, id) < 0) {
                                self.selectOrder.push(id);
                                $target.find('.icon-checkbox').html('<div class="cb--order" data-cb-selected-order="' + self.selectOrder.length + '">' + self.selectOrder.length + '</div>');
                            }
                        }
                    }

                    if (self.autoComplete) {
                        self.recalculate();
                    } else {
                        notSelected = self.$el.find(self.cbIdentifier).not('.is-selected').length;

                        if (self.cbType == 'order') {
                            if (notSelected == 0) {
                                //no checkboxes selected
                                self.submitButton.removeClass('is-disabled').attr("aria-disabled",false);
                            } else {
                                self.submitButton.addClass('is-disabled').attr("aria-disabled",true);
                            }

                            if (totalCheckboxes == notSelected) {
                                if (self.resetButton) {
                                    self.resetButton.addClass('is-disabled').attr("aria-disabled",true);
                                }
                            } else {
                                if (self.resetButton) {
                                    self.resetButton.removeClass('is-disabled').attr("aria-disabled",false);
                                }
                            }
                        } else {
                            if (totalCheckboxes == notSelected) {
                                //no checkboxes selected
                                self.submitButton.addClass('is-disabled').attr("aria-disabled",true);
                                if (self.resetButton) {
                                    self.resetButton.addClass('is-disabled').attr("aria-disabled",true);
                                }
                            } else {
                                //some checkboxes are selected
                                self.submitButton.removeClass('is-disabled').attr("aria-disabled",false);
                                if (self.resetButton) {
                                    self.resetButton.removeClass('is-disabled').attr("aria-disabled",false);
                                }
                            }
                        }

                    }
                };


            self.selectOrder = [];

            if (checkboxes.length == 0) {
                console.error("Unable to load checkboxes. no data-cbids exists");
            }



            _.each(checkboxes, function(cb) {
                var $cb = $(cb),
                    id = $cb.data('cbid'),
                    showIfCorrect = $cb.data('showIfCorrect'),
                    showIfWrong = $cb.data('showIfWrong');

                //load the assist text for the checkboxes
                //$cb.find(".a11y-assist-text").html("Not Selected");

                
                if(showIfCorrect){
                    $(showIfCorrect).hide();
                }
                if(showIfWrong){
                    $(showIfWrong).hide();
                }

                $cb.on("click", function(e) {
                    e.target = (this);
                    clickHandler(e);
                });

                if (reload) {
                    self.definedOrder.push({
                        id: id,
                        order: ($cb.data('order') || i++)
                    });
                    self.checkboxes.push({
                        id: $(cb).data('cbid'),
                        $el: $cb,
                        weight: ($(cb).data('weight') || 1)
                    });

                }
            });

            self.definedOrder = _.sortBy(self.definedOrder, function(o) {
                return o.order;
            });
            self.totalWeightage = (_.reduce(self.checkboxes, function(memo, cb) {
                return cb.weight + memo;
            }, 0));

        },
        randomize: function() {
            var self = this;

            if (self.checkboxes.length == 0) {
                return false;
            }

            self.$el.empty();
            self.checkboxes = _.shuffle(self.checkboxes);

            _.each(self.checkboxes, function(cb) {
                self.$el.append(cb.$el);
            });

            self.loadCheckboxes(false);
        },
        load: function($el) {
            this.$el = $el;

            var data = this.$el.data(),
                self = this,
                allowPartials = false;

            this.loadCheckboxes();

            if (_.has(data, 'cbType') && ['multiple', 'order'].indexOf(data.cbType) >= -1) {
                self.cbType = data.cbType;
            }



            if (!(_.has(data, 'correct'))) {
                //Always load neutral response feedback
                self.showNeutralFeedback = true;
                console.log("[Checkbox Message] Checkboxes initialized to show neutral feedback always");
            } else {
                self.correct = ((data.correct + "") || "").split(',');
            }

            if (!_.has(data, 'submit')) {
                self.autoComplete = true;
                console.log("[Checkbox Message] No submit button mentioned. Checkbox will auto complete");
            } else {
                self.submitButton = $(data.submit);
                self.submitButton.addClass('is-disabled').attr("aria-disabled",true);
                self.submitButton.on("click", self.recalculate);
            }

            if (_.has(data, 'reset')) {
                self.resetButton = $(data.reset);
                self.resetButton.removeClass('is-disabled').attr("aria-disabled",false);
                self.resetButton.on("click", function(e) {
                    if ($(e.currentTarget).hasClass('is-disabled')) {
                        return false;
                    }

                    self.submitButton.addClass('is-disabled').attr("aria-disabled",true);
                    self.resetButton.addClass('is-disabled').attr("aria-disabled",true);

                    self.selectOrder = [];
                    if (self.cbType == 'order') {
                        $('[data-cb-selected-order]').remove();

                    }

                    self.$el.find(self.cbIdentifier)
                        .removeClass('is-selected')
                        .removeClass('is-correct')
                        .removeClass('is-wrong');
                });

                self.resetButton.addClass('is-disabled').attr("aria-disabled",true);
            }


            self.animateOrder = (_.has(data, 'animateOrder') && data.animateOrder == 'showHide') ? data.animateOrder : "hideShow";

            if (_.has(data, 'partialOn')) {
                allowPartials = (data.partialOn == '1');
            }

            if (_.has(data, 'animations')) {
                self.animations = data.animations;
            }
            if (_.has(data, 'animation')) {
                self.animations = data.animation;
            }

            if (allowPartials && _.has(data, 'partials') && data.partials) {
                self.partials.type = data.partialType || 'of';
                self.partials.tgt = ("" + (data.partials || "")).split(',');
            }

            if (_.has(data, 'randomize') && data.randomize) {
                this.randomize();
            }


            if (!_.has(data, 'feedbackShowSelector')) {
                console.warn("[CHECKBOXES] NO FEEDBACK SELECTOR IN DATA TAG>. NO FEEDBACKS WILL BE SHOWN BY THE CHECKBOX");
            } else {
                self.feedbacks.show = $(data.feedbackShowSelector);
                if (self.feedbacks.show.length == 0) {
                    console.warn("[CHECKBOXES] FEEDBACK NOT LOCATED");
                }
                self.feedbacks.hide = $(data.feedbackHideSelector);



                self.hide(self.feedbacks.show.find("[data-caf], [data-waf], [data-naf], [data-pcaf]"));
                self.hide(self.feedbacks.show);
                //self.feedbacks.show.find("[data-caf], [data-waf], [data-naf], [data-pcaf]").hide();
                //self.feedbacks.show.hide();
            }

            if (_.has(data, 'preventTrack') && data.preventTrack) {
                self.trackable = false;
            }


        }
    });


    window.bazingaApp.views.checkboxView = window.bazingaApp.views.abstractComponentView.extend({
        el: '[data-checkbox]:not(.js-prevent-interaction)',
        cbEl: '.checkbox',
        feedbacks: null,
        allow: {
            randomize: true
        },
        cbList: [],
        events: {

        },
        initialize: function() {
            if (this.$el.length == 0) {
                this.enabled = false;
            }

            this.componentType = 'checkboxes';
            var self = this,
                recalculate = function(e) {
                    var todo = 0;
                    _.each(self.trackables, function(v) {
                        if (v.getIsCompleted()) {
                            return false;
                        }

                        todo++;
                    });

                    if (todo == 0) {
                        self.completed = 'completed';
                        //trigger event for appview
                        self.trigger('completed:checkboxes', self, e);
                    }
                };

            this.trackables = [];


            _.each(this.$el, function(el) {
                var $el = $(el),
                    cbManager = new window.bazingaApp.views.checkboxManager();
                cbManager.on("completed:checkbox", function(e) {
                    recalculate(e);
                });
                cbManager.load($el);

                if (cbManager.getIsTrackable()) {
                    self.trackables.push(cbManager);
                }

                self.cbList.push(cbManager);
            });



        },
        setAllow: function(allow) {
            this.allow = _.extend(this.allow, allow);
            return this;
        }

    });
})(window.$ || window.JQuery);

(function ($) {
    "use strict";

    /**
     *	display tick
     * 	
     * 	Mandatory Attributes:
     * 	data-accordion-type="tick" (the address the tick element by backbone, bind view on it)
     * 
     * 	useage:
     * 
     * <a class="row" data-accordion-type="tick">
     *     <span class="col-xs-10">...</span>
     *     <span class="col-xs-2 u-text-right">
     *       <span class="icon-tick"></span> //add class "hidden" in here to set default visibility if necessary
     *     </span>
     * </a>
     *
     */
    window.bazingaApp.views.tickView = window.bazingaApp.views.abstractComponentView.extend({
        el: '[data-accordion-type="tick"]:not(.js-prevent-interaction)',
        events: {
            "click": "show"
        },
        selector: {
        	container: '[data-accordion-type="tick"]'
        },

        initialize: function () {
            _.bindAll(this, 'show','recalculate');
            if(this.$el.length == 0){this.enabled = false;}

            this.componentType='accordionTick';
            this.recalculate();
        },
        recalculate: function () {
            if(this.getIsCompleted()) return this;

            var self = this,
                completed = self.completed = self.completed || false,
                totalCount = $(self.selector.container).length,
                incompleteCount = $(self.selector.container).not(".is-visited").length;

            if( incompleteCount == 0){
                self.completed = 'completed';
                self.trigger('completed:tick',self);
            }
            else if(incompleteCount !== totalCount){
                self.completed = 'incomplete';
            }

            return self;
        },
        show: function(e) {
        	var target = e.target,
        		self = this,
                $target = $(target).data('accordionType')?$(target):$(target).closest(self.selector.container);
            
            e.preventDefault();
            //selecting the current target
            $(self.selector.container).removeClass('is-selected');
            //complete the target
            $target.addClass('is-selected').addClass('is-visited');
           
//           $('.icon-tick',$target).removeClass('hidden');
           $('.panel-heading .icon-tick',$target).show();
           self.recalculate();
        }
    });
})(window.$ || window.JQuery);
(function($){
    "use strict";

    /**
     * @todo: convert to a template . So that the preloader gets compiled
     * @type {*}
     */
    window.bazingaApp.views.audioView = window.bazingaApp.views.abstractComponentView.extend({
        el:$("audio:not(.js-prevent-interaction)"),

        initialize : function(){
            this.componentType='audio';
        },
        load:function(){
            var deferred = $.Deferred(),
                transcript = $(".js-transcript"),
                self=this,
                type='onStart',
                data,
                onComplete = function(){
                    if(!self.getIsCompleted()){
                        deferred.resolve();
                        self.completed = "completed";
                        self.trigger('completed:audio',self);
                    }
                };

            if(this.$el.length == 0 || typeof this.$el.mediaelementplayer !== 'function'){
                this.enabled=false;
                setTimeout(function(){
                    deferred.fail();
                },200);
            }
            else{
                data= this.$el.data();
                data.features = data.features ? (data.features.split(',')): ['playpause','progress','volume'];
                if(data.mediaType) {type = data.mediaType;}


                _.defaults(data , {
                    success: function (media, node, player) {
                        // do something when audio starts
                        // add event listener
                        if(type == 'onFinish'){
                            media.addEventListener('ended', onComplete);
                        }
                        else{
                            media.addEventListener('playing',onComplete);
                        }

                    }
                });


                self.player = this.$el.mediaelementplayer(data);
				
				//@since 1.0 optional audio
				if(self.player.hasClass("optional")){
					onComplete();	
				}

                // Transcript Button
                if ($(transcript).length) {
                    transcript.on("click", function() {
                        onComplete();
                        if (transcript.hasClass("is-enabled")) {
                            transcript.removeClass("is-enabled");
                            $(".transcript").slideUp();
                        } else {
                            transcript.addClass("is-enabled");
                            $(".transcript").slideDown();
							$(".transcript-focus").attr("tabindex",0).focus();
                        }
                    })
                }
            }

            return deferred.promise();
        },

        play:function(){
            if(this.player.length > 0){
                var player = _.first(this.player);
                player.play();
            }
        },
        pause : function(){
            if(this.player.length > 0){
                var player = _.first(this.player);
                player.player.pause();
            }
        },
        resetSource : function(src){
            if(this.player.length > 0){
                var player = _.first(this.player);
                player.pause();
                player.setSrc(src);
                player.play();
            }
        },
        enable : function(){
            this.$el.show();
            return this;
        },
        disable : function(){
            this.$el.hide();
            return this;
        }

    });

    window.bazingaApp.views.videoView = window.bazingaApp.views.abstractComponentView.extend({
        el:$("video:not(.js-prevent-interaction)"),
        initialize : function(){
            this.componentType='video';
        },
        load:function(){
            var deferred = $.Deferred(),
                transcript = $(".js-transcript"),
            	data,
                self=this,
                type='onStart',
                data,
                onComplete = function(){
                    if(!self.getIsCompleted()){
                        deferred.resolve();
                        self.completed = "completed";
                        self.trigger('completed:video',self);
                    }
                };

            if(this.$el.length == 0 || typeof this.$el.mediaelementplayer !== 'function'){
                this.enabled=false;
                setTimeout(function(){deferred.fail();},200);
            }
            else{
            	data= this.$el.data();
                data.features = data.features ? (data.features.split(',')): ['playpause','progress','fullscreen'];
                type = data.mediaType || 'onStart';

                _.defaults(data , {
                    success: function (media, node, player) {
                        // do something when audio starts
                        // add event listener
                        if(type == 'onFinish'){
                            media.addEventListener('ended', onComplete);

                        }
                        else{
                            media.addEventListener('playing',onComplete);
                        }

                    }
                });
                self.player = this.$el.mediaelementplayer(data);

                console.log(self.player);
                // Transcript Button
                if ($(transcript).length) {
                    transcript.on("click", function() {
                        onComplete();
                        if (transcript.hasClass("is-enabled")) {
                            transcript.removeClass("is-enabled");
                            $(".transcript").slideUp();
                        } else {
                            transcript.addClass("is-enabled");
                            $(".transcript").slideDown();
                        }
                    })
                }
            }


            return deferred.promise();
        },
        play:function(){
            if(this.player.length > 0){
                var player = _.first(this.player);
                player.play();
            }
        },
        pause : function(){
            if(this.player.length > 0){
                var player = _.first(this.player);
                player.player.pause();
            }
        },
        resetSource : function(src){
            if(this.player.length > 0){
                var player = _.first(this.player);
                player.pause();
                player.setSrc(src);
                player.play();
            }
        },
        enable : function(){
            this.$el.show();
            return this;
        },
        disable : function(){
            this.$el.hide();
            return this;
        }

    });
})(window.$||window.JQuery);
(function ($) {
    "use strict";
    window.bazingaApp.views.fillupManager = window.bazingaApp.views.abstractComponentView.extend({
        fillups : [],
        fuIdentifier : '[data-fu-id]',


        showNeutralFeedback : false,
        autoComplete : false,
        fuType : 'single',
        trackable : true,
        partials : {
            type :'of',
            tgt  : null
        },
        correct : [],
        totalWeightage : 0,
        state : "not-attempted",
        getState : function(){return this.state;},
        getIsTrackable : function(){return this.trackable ;},
        initialize : function(){
            _.bindAll(this,'load','randomize','loadfillups','recalculate','show','hide');
        },
        selectOrder : [],
        definedOrder : [],
        feedbacks : {show:null,hide:null},
        animations : 'fade',

        recalculate : function(e){

            if(e && e.currentTarget && ($(e.currentTarget).hasClass('is-disabled'))){
                e.stopPropagation();
                return false;
            }

            $(e.currentTarget).addClass('is-disabled');

            var self=this,
                allFillUps = self.fillups,
                answers = {"correct" : [], "wrong" : [],"neutral":[],"partial":[]},
                score= 0,
                score_max = self.totalWeightage,
                allIds=[],
                i= 0,
                fb = 'naf';



            if(self.resetButton){self.resetButton.addClass('is-disabled').attr("aria-disabled",true);}


            _.each(allFillUps,function(fu){
                var $el = fu.$el,
                    selected = $el.val(),
                    correct,
                    allCorrect=$el.find('[data-correct]'),
                    correctValue=[],
                    id = $el.data('fuId'),
                    intersection;

                $el.addClass('is-disabled').attr('disabled','true');

                if(allCorrect.length == 0){
                    $el.addClass('is-neutral');
                    answers.neutral.push({id:id,$el:$el});
                    //Always a neutral feedback
                    return true;
                }

                _.each(allCorrect, function(k){
                    correctValue.push($(k).val());
                });

                if(typeof selected == 'string'){selected = [selected];}

                intersection= _.intersection(selected,correctValue);

                if(intersection.length == correctValue.length){
                    //correct
                    answers.correct.push({id:id,$el:$el});
                    $el.addClass('is-correct');
                    score += parseInt( $(fu).data('weight') || 1);
                }
                else if(intersection.length > 0){
                    //partially right
                    answers.partial.push({id:id,$el:$el});
                    $el.addClass('is-partial');
                    score += (parseInt( $(fu).data('weight') || 1))*0.5;
                }
                else{
                    //wrong
                    answers.wrong.push({id:id,$el:$el});
                    $el.addClass('is-wrong');
                }

                allIds.push(id);
            });

            var ret = {
                target:self,
                answers:answers,
                allIds : allIds,
                score:score,
                score_max:score_max,
                percentage : ((score/score_max)*100)
            };


            ret.state="passed";

            if(allIds.length == answers.correct.length){
                ret.state = self.state = "passed";
            }
            else{
                console.log(self.partials);
                //trigger neutral
                if(self.partials.tgt){
                    if(self.partials.type == 'of'){
                        if(answers.correct.length >= self.partials.tgt[0]){
                            ret.state = 'partial';
                        }
                        else{
                            ret.state = 'failed';
                        }
                    }
                    else if(self.partials.type == 'in'){
                        var overlap = _.intersection(self.partials.tgt,answers.correct);
                        if(!_.isEmpty(overlap) && (overlap.length == self.partials.tgt.length )){
                            ret.state = 'partial';
                        }
                        else{
                            ret.state = 'failed';
                        }
                    }
                    else{
                        if(ret.percentage >= parseInt((self.partials.tgt[0] || 50))){
                            ret.state = 'partial';
                        }
                        else{
                            ret.state = 'failed';
                        }
                    }
                }
                else{
                    ret.state = 'failed';
                }
            }


            fb = '[data-caf]';
            if(self.showNeutralFeedback){
                fb='[data-naf]';
            }
            else {
                if(ret.state=='passed'){
                    fb='[data-caf]';
                }
                else if(ret.state=='failed'){
                    fb='[data-waf]';
                }
                else{
                    fb='[data-pcaf]';
                }
            }

            if(self.feedbacks.show){
                self.show(self.feedbacks.show.find(fb,null,100));

                var showFeedback = function(callback){
                        self.show(self.feedbacks.show,callback);
                    },
                    hideFeedback = function(callback){
                        if(self.feedbacks.length == 0){
                            callback();
                        }
                        else{
                            self.hide(self.feedbacks.hide,callback);
                        }
                    };


                if(self.animateOrder =='hideShow'){

                    hideFeedback(function(){
                        showFeedback(function(){
                            $(".tip").addClass('hidden');
                            self.trigger("completed:fillup",ret);
                        });
                    });
                }
                else{
                    showFeedback(function(){
                        $(".tip").addClass('hidden');
                        hideFeedback(function(){
                            self.trigger("completed:fillup",ret);
                        });
                    });
                }
            }
            else{
                setTimeout(function(){self.trigger("completed:fillup",ret)},20);
                console.warn("No feedbacks located. SO event was triggered");
            }

            console.log(ret);
            self.completed = 'completed';
      },

        show : function(element,callback,delay){
            var self=this,
                promise,
                classOverride = element.data('animationHide') || self.animations;

            delay = delay || 300;


            if(typeof callback !== 'function'){
                callback =function(){}
            }

            switch (classOverride){
                case "fade":
                case "fadeinout":
                    promise = element.fadeIn(delay,callback);
                    break;

                case "slideUp":
                case "slideUpDown":
                    promise = element.slideUp(delay,callback);
                    break;

                case "slideDown":
                case "slideDownUp":
                    promise = element.slideDown(delay,callback);
                    break;

                default:
                    promise = element.show(delay,callback);
                    break;
            }

            return promise;
        },
        hide : function(element,callback,delay){
            var self=this,
                promise,
                classOverride = element.data('animationHide') || self.animations;

            delay = delay || 600;

            if(typeof callback !== 'function'){
                callback =function(){}
            }


            switch (classOverride){
                case "fade":
                case "fadeinout":
                    promise = element.fadeOut(delay,callback);
                    break;

                case "slideUp":
                case "slideDownUp":
                    promise = element.slideUp(delay,callback);
                    break;

                case "slideDown":
                case "slideUpDown":
                    promise = element.slideDown(delay,callback);
                    break;

                default:
                    promise = element.hide(delay,callback);
                    break;
            }

            return promise;
        },
        loadfillups : function(reload){
            reload = reload===false ? false : true;
            if(reload){this.fillups = [];};

            var self=this,
                i= 1,
                fillups = this.$el.find(this.fuIdentifier);

            if(fillups.length == 0){
                console.error("Unable to load fillups. no data-fuids exists");
            }

            _.each(fillups, function (fu) {
                var $fu = $(fu),
                    id = $fu.data('fuId'),
                    type = $fu.data('dropDown') || 'selectpicker';


                if (reload) {
                    self.definedOrder.push({id: id, order: ($fu.data('order') || i++)});
                    self.fillups.push({
                        id: id,
                        $el: $fu,
                        weight: ($fu.data('weight') || 1)
                    });

                    if(type == 'selectpicker'){
                        $fu.selectpicker();
                    }
                }
            });

            self.definedOrder = _.sortBy(self.definedOrder, function(o){return o.order;});
            self.totalWeightage = (_.reduce(self.fillups,function(memo,fu){return fu.weight+memo;},0));
        },
        randomize: function(){
            var self=this,
                randomizeOptions  = function($el){
                    if(!$el || $el.length == 0){return false;}


                    _.each($el,function(elem){
                        var $elem = $(elem),
                            options = $elem.find('option');

                        $elem.empty();
                        options = _.shuffle(options);

                        _.each(options,function(op){
                            $elem.append($(op));
                        });
                    });

                    return $el;
                };  

            if(self.fillups.length == 0){return false;}

            _.each(self.fillups,function(fillup){
                var hasOptGroup = fillup.$el.find('optgroup').length;

                if(hasOptGroup){
                    randomizeOptions(fillup.$el.find('optgroup'));
                }
                else{
                    randomizeOptions(fillup.$el);
                }
            });
            self.loadfillups(false);
        },
        load : function($el){
            this.$el = $el;

            var data = this.$el.data(),
                self=this,
                allowPartials = false;

            this.loadfillups();

            if(_.has(data,'fillUpType') && ['multiple'].indexOf(data.fillUpType) >= -1){
                self.fuType= data.fuType;
            }


            //@todo Correct has to be loaded for each fill up

            if(!_.has(data,'submit')){
                console.error("[fillup Message] No submit button mentioned. A submit button is necessary for evaluation.");
            }
            else{
                self.submitButton = $(data.submit);
                self.submitButton.on("click",self.recalculate);
            }


            if(_.has(data,'reset')){
                self.resetButton = $(data.reset);
                self.resetButton.removeClass('is-disabled').attr("aria-disabled",false);
                self.resetButton.on("click",function(){
                    self.submitButton.addClass('is-disabled').attr("aria-disabled",true);
                });
            }


            self.animateOrder = (_.has(data,'animateOrder')  && data.animateOrder == 'showHide') ? data.animateOrder : "hideShow";


            if(_.has(data,'animations')){
                self.animations = data.animations;
            }
            if(_.has(data,'animation')){
                self.animations = data.animation;
            }

            //Partial set up
            if(_.has(data,'partialOn')){
                allowPartials = (data.partialOn == '1');
            }
            if(allowPartials && _.has(data,'partials') && data.partials){
                self.partials.type = data.partialType || 'of';
                self.partials.tgt = (""+(data.partials || "")).split(',');
            }

            if(!_.has(data,'randomize') ||  (data.randomize !== '0') ){
                //@todo : Randomize  to be programmed
                this.randomize();
            }

            if(!_.has(data,'feedbackShowSelector')){
                console.warn("[fillups] NO FEEDBACK SELECTOR IN DATA TAG>. NO FEEDBACKS WILL BE SHOWN BY THE fillup");
            }
            else{
                self.feedbacks.show = $(data.feedbackShowSelector);
                if(self.feedbacks.show.length == 0){
                    console.warn("[fillups] FEEDBACK NOT LOCATED");
                }
                self.feedbacks.hide = $(data.feedbackHideSelector);


                self.hide(self.feedbacks.show.find("[data-caf], [data-waf], [data-naf], [data-pcaf]"));
                console.log(self.feedbacks.show.find("[data-caf], [data-waf], [data-naf], [data-pcaf]"));
                self.hide(self.feedbacks.show);
                self.feedbacks.show.find("[data-caf], [data-waf], [data-naf], [data-pcaf]").hide();
                self.feedbacks.show.hide();
            }

            if(_.has(data,'preventTrack') && data.preventTrack){
                self.trackable = false;
                self.completed = 'completed';
                self.trigger("completed:fillup",self)
            }
        }
    });


    window.bazingaApp.views.fillupView = window.bazingaApp.views.abstractComponentView.extend({
        el: '[data-fillup]:not(.js-prevent-interaction)',
        fuEl : '.fillup',
        feedbacks : null,
        allow : {
            randomize : true
        },
        fuList : [],
        events: {

        },
        initialize: function () {
            if(this.$el.length == 0){this.enabled = false;}

            this.componentType='fillups';
            var self=this,
                recalculate = function(e){
                    var todo  = 0;
                    _.each(self.trackables, function(v){
                        if(v.getIsCompleted()){return false;}
                        todo++;
                    });

                    if(todo ==0){
                        self.completed = 'completed';
                        //trigger event for appview
                        self.trigger(('completed:'+self.componentType) ,self,e);
                    }
                };

            this.trackables = [];


            _.each(this.$el, function(el){
                var $el = $(el),
                    fuManager = new window.bazingaApp.views.fillupManager();

                fuManager.on("completed:fillup",function(e){recalculate(e);});
                fuManager.load($el);

                if(fuManager.getIsTrackable()){self.trackables.push(fuManager);}
                self.fuList.push(fuManager);
            });
        },
        setAllow : function ( allow){
            this.allow = _.extend(this.allow,allow);
            return this;
        }

    });
})(window.$ || window.JQuery);
(function($) {
    "use strict";

    window.bazingaApp.views.carouselView = window.bazingaApp.views.abstractComponentView.extend({
        el: '[data-ride="carousel"]:not(.js-prevent-interaction)',
        events: {},
        clickable: true,
        selector: {},
        load: function() {
            var loadedFirstTime = false;
            var self = this,
                getItemIndex = function(item) {
                    self.$items = item.parent().children('.item')
                    return self.$items.index(item)
                },
                isComplete = function($tgt) {
                    var $active = self.$el.find('.item.active'),
                        ix = getItemIndex($active),
                        focussible;
                    //@since 1.0
                    if (self.$el.data('accessible') == true) {
                        //version 1.2.0 +
                        console.log("Custom module carousel");
                        //This is an accessible carousel for CUstom modules
                        if (ix == 0) {
                           self.$el.find('.prevContainer').css('display','none');
                        } else {
                            self.$el.find('.prevContainer').css('display','block');
                        }

                        if (ix == (self.$items.length - 1)) {
                            //complete carousel
                            self.completed = "completed";
                            self.trigger('completed:carousel', self);

                            if (self.isWrap) {
                                self.$el.find('.nextContainer').css('display','none');
                            }
                        } else {
                            if (self.isWrap) {
                                self.$el.find('.nextContainer').css('display','block');
                            }
                        }

                        focussible = $active.find('.js-carousel-focus');

                        //don't
                        if (loadedFirstTime) {
                            if (focussible.length > 0) {
                                $(focussible[0]).attr("tabindex", 0);
                                focussible[0].focus();
                            }
                        }
                        if (!loadedFirstTime) {
                            loadedFirstTime = true;
                        }
                    } else {
                        //AEC modules
                        if (ix == 0) {
                            self.$el.find('[data-slide="prev"]').hide();
                            self.$el.find('[data-slide="next"]').focus();
                        } else {
                            self.$el.find('[data-slide="prev"]').show();
                        }

                        if (ix == (self.$items.length - 1)) {
                            self.completed = "completed";
                            self.trigger('completed:carousel', self);
                            if (self.isWrap) {
                                self.$el.find('[data-slide="next"]').hide();
                                self.$el.find('[data-slide="prev"]').focus();
                            }
                        } else {
                            if (self.isWrap) {
                                self.$el.find('[data-slide="next"]').show();
                            }
                        }
                    }


                };


            this.componentType = 'carousal';
            if (this.$el.length == 0) {
                this.enabled = false;
                this.completed = 'completed';
            } else {
                this.completed = "incomplete";
            }

            self.isWrap = this.$el.data('wrap');

            _.each(this.$el, function(e) {
                isComplete($(e));
            });



            this.$el.on('slid.bs.carousel', function() {
                isComplete();
            });


            this.recalculate();
        },
        recalculate: function() {}
    });
})(window.$ || window.JQuery);

(function ($) {
    "use strict";
	/**
     * checkpoint module. Different options will bring you to different path.
     * complete the page once one of the option has been choosed.
     *
     */
    window.bazingaApp.views.checkPointView = window.bazingaApp.views.abstractComponentView.extend({
        el: $('[data-type=checkpoint]'),
        events: {
            "click": "recalculate"
        },
        initialize: function () {
            _.bindAll(this, 'recalculate');
            if(this.$el.length == 0){
                this.enabled = false;
                this.completed= 'completed';
            }
            else{
                this.completed = "incomplete";
            }
        },
        recalculate: function (e) {
            var self = this,
                completed = self.completed = self.completed || false,
                el = $(e.target),
                forwardBtn = $('.footbar [data-goto]:eq(0)');
			
			if(forwardBtn.length == 0){return;}
			
			//remove all "is-selected" class from all other options except itself
			self.$el.not(e.target).removeClass('is-selected');
			el.addClass('is-selected');
			
			self.completed = 'completed';
			self.trigger('completed:checkpoint',self);
			
			forwardBtn.attr('data-goto',el.attr('data-checkpoint-goto'));
			
            return self;
        }
    });
})(window.$ || window.JQuery);
(function($){
    "use strict";

    window.bazingaApp.models.API = Backbone.Model.extend({
        'load': function(courseConfig){
            var self=this,
                deferred = $.Deferred();

            self.courseConfig = courseConfig;


            window.bazingaApp.models.scorm12 = self.scorm12 = new  window.bazingaApp.models.scormDataModel();
            self.scormAPI =  window.bazingaApp.models.scorm12.get('API');

            window.bazingaApp.models.data= this.data = new window.bazingaApp.models.dataModel();
            this.data.setAPI(self.scormAPI);
            self.trigger("initialize:dataModel",self);

            this.data
                .fetch()
                .always(function(dataModelContext){
                    if(window.bazingaApp.models.data.getLessonStatus()){
                        //Lets start the course
                        window.bazingaApp.models.data.setLessonStatus('incomplete');
                    }
                    // initialize
                    self.scormAPI.LMSCommit("");

                    self.trigger("initialize:api",self);
                    //load the navigation/
                    window.bazingaApp.models.navigation = self.navigation = new window.bazingaApp.models.navigationModel();

                    window.bazingaApp.models.navigation.once('initialize:handshake', function () {
                        self.trigger('initialize:handshake',self);
                    });
                    window.bazingaApp.models.navigation.once('initialize:suspendData', function () {
                        self.trigger('initialize:suspendData',self);
                    });
                    window.bazingaApp.models.navigation.once('initialize:page', function () {
                        self.trigger('initialize:page',self);
                    });


                    window.bazingaApp.models.navigation
                        .setDataModel(dataModelContext)
                        .setCourseConfig(courseConfig)
                        .load(self.courseConfig.navigation.src)
                        .always(function(navModelContext){
                            self.trigger("initialize:navigation",self);
                            //initializing nav api
                            deferred.resolve(self);
                        });
                });



            return deferred.promise();
        },

        setCourseStatus :  function(status){return window.bazingaApp.models.navigation.setCourseStatus(status);},
        setPageScore :  function(score){return window.bazingaApp.models.navigation.setPageScore(score);},
        setPageInfo : function(info){window.bazingaApp.models.navigation.setInfo(info); return self;},
        setPageCompletion : function(completion){window.bazingaApp.models.navigation.setPageCompletion(completion); return self;},
        setPageData : function(data){window.bazingaApp.models.navigation.setPageData(data); return self;},
        reattemptScreen : function(screen,role){return window.bazingaApp.models.navigation.reattemptScreen(screen,role);},
        getPageScore :  function(){return window.bazingaApp.models.navigation.getPageScore();},
        setCourseScore :  function(score){return window.bazingaApp.models.navigation.setCourseScore(score);},
        setMultiple :  function(values){return window.bazingaApp.models.navigation.setMultiple(values);},
        setCourseScoreFrom :  function(score){return window.bazingaApp.models.navigation.setCourseScoreFrom(score);},
        getPageCompletion : function(){return window.bazingaApp.models.navigation.getPageCompletion();},
        getPageData : function(){return window.bazingaApp.models.navigation.getPageData();},
        getLockables : function(){return window.bazingaApp.models.navigation.getLockables();},
        getPageCount : function(){return window.bazingaApp.models.navigation.getPageCount();},
        getAllPages : function(){return window.bazingaApp.models.navigation.getAllPages();},
        getInfo : function(info){return window.bazingaApp.models.navigation.getInfo(info);},
        getPreviousScreen : function(){return window.bazingaApp.models.navigation.getPreviousScreen();},
        getPageDataForPage : function(){return window.bazingaApp.models.navigation.getPageDataForPage();},
        getConfigForScreen :function(){return window.bazingaApp.models.navigation.getConfigForScreen();},
        getNavForScreen :function(id){return window.bazingaApp.models.navigation.getNavForScreen(id);},
        getNavForDialog :function(id){return window.bazingaApp.models.navigation.getNavForDialog(id);},
        getNavForScreenByRole :function(role,id){return window.bazingaApp.models.navigation.getNavForScreenByRole(role,id);},
        getDecision : function(decisionPoint,role){return window.bazingaApp.models.navigation.getDecision(decisionPoint,role);},
        getRole : function(){return window.bazingaApp.models.navigation.getRole();},
        getIsDialog : function(){return window.bazingaApp.models.navigation.getIsDialog();},
        getCrumbs : function(){return window.bazingaApp.models.navigation.getCrumbs();},
        getNav : function(){return window.bazingaApp.models.navigation.getNav();},
        getSuspendData : function(){return window.bazingaApp.models.navigation.getSuspendData();},
        setPrimaryScore : function(score){return window.bazingaApp.models.navigation.setPrimaryScore(score);},
        getPrimaryScore : function(score){return window.bazingaApp.models.navigation.getPrimaryScore();},
        setSecondaryScore : function(score){return window.bazingaApp.models.navigation.setSecondaryScore(score);},
        getSecondaryScore : function(score){return window.bazingaApp.models.navigation.getSecondaryScore();},
        setTertiaryScore : function(score){return window.bazingaApp.models.navigation.setTertiaryScore(score);},
        getTertiaryScore : function(score){return window.bazingaApp.models.navigation.getTertiaryScore();},
        setCourseData : function(data){return window.bazingaApp.models.navigation.setCourseData(data);},
        getCourseData : function(){return window.bazingaApp.models.navigation.getCourseData();},
        setCurrentPool : function(poolId){return window.bazingaApp.models.navigation.setCurrentPool(poolId);},
        getCurrentPool : function(){return window.bazingaApp.models.navigation.getCurrentPool();},
        getCourseConfig : function(){return this.courseConfig;},
        getScorm12DataModel : function(){return this.scorm12;},
        getScormAPI : function(){return this.scormAPI;},
        getData : function(){return this.data;},
        getNavigation : function(){return this.navigation;},

        shutDown : function(){
            var self=this;
            window.bazingaApp.models.scorm12.switchTimers();

            window.bazingaApp.models.data.commit("");
            window.bazingaApp.models.scorm12
                .finishSCO()
                .always(function(){
                    //Navigate toe exit page
                    window.location.replace(self.courseConfig.finishPage);
                });
        }

    });

})(window.$||window.JQuery);
(function(){
    window.bazingaApp.views.launcherView = Backbone.View.extend({
        el:$('#js-app'),
        events: {
        },
        /**
         * Load the application
         */
        load : function(){
            console.log("Loading launcher view");
            self.backstretch = new window.bazingaApp.views.backstretchView();
            self.preloader = new window.bazingaApp.views.preloaderView();
            self.preloader.setLabel('Loading..').setPercentage(0);
            self.popup = null;


            console.log(location.protocol + "//" + location.host);
            $.getJSON("../config/config.json",function(courseConfig){
                courseConfig = _.defaults(courseConfig, {
                    "launchPage" : "pages/splash.html",
                    "popup" : false
                });
                self.courseConfig=courseConfig;

                console.log("Launcher loading");
                console.log("Settting local storage");

                if(self.courseConfig.popup){
                    self.preloader
                        .incrementTo(100,10)
                        .always(function(){
                            var props = "width=1012,height=688,resizable=no,status=no,toolbar=no,menubar=no,location=no",
                                check;

                            self.popup = window.open(self.courseConfig.launchPage,'savve-popup', props);


                            check  = function(){
                                setTimeout(function(){
                                    if(self.popup && !self.popup.closed){
                                        check();
                                    }
                                    else{
                                        window.location.replace(courseConfig.finishPage || 'finish.html');
                                    }
                                },1000);
                            };

                            if (window.focus) {self.popup.focus()}
                            check();
                        });
                }
                else{
                    window.location.replace((courseConfig.launchPage || 'splash.html'));
                    window.localStorage.setItem("firstLaunch",true);
                }
            });
        }
    });
}());

(function () {
    window.bazingaApp.views.appView = Backbone.View.extend({
        el: $('#js-app'),
        events: {
            "click [data-decision]": "gotoDecisionPoint",
            "click [data-goto]": "gotoDecisionById",
            "click [data-trigger]": "triggerData",
            "click [data-reattempt]": "reattempt",

            "touchend [data-decision]": "gotoDecisionPoint",
            "touchend [data-goto]": "gotoDecisionById",
            "touchend [data-trigger]": "triggerData",
            "touchend [data-reattempt]": "reattempt",
            "keydown" : "keydown"
        },
        menu: {
            $el: $('#js-dropdown'),
            tabIndex: 4
        },

        initializeEvents : function($el){
            var self=this;
            $el.find('[data-decision]').off("click touchend").on("click touchend",self.gotoDecisionPoint);
            $el.find('[data-goto]').off("click touchend").on("click touchend",self.gotoDecisionById);
            $el.find('[data-trigger]').off("click touchend").on("click touchend",self.triggerData);
        },

        reloadParentEvents : function(){
            //hack. this had to happen because the popup triggers were switching events off
            $('[data-goto]').on("click touchend",self.gotoDecisionById);
            $('[data-decision]').on("click touchend",self.gotoDecisionPoint);
            $('[data-trigger]').on("click touchend",self.triggerData);
        },
        /**
         * List of all interactions on screen
         * When there are interactions like checkboxes, they are automatically recognized.
         * Unless a force_screen_on_complete is true, you have to complete all interactions on screen before the screen completes
         */
        interactions: [],
        initialize: function () {
            var self = this;

            _.bindAll(this, 'goNext', 'gotoDecisionPoint', 'gotoDecisionById', 'triggerData', 'goto',
                'goBack', 'getBreadCrumbs', 'initializePageComponents', 'populateMenu', 'initializeAccessibility',
                'initializeMenu', 'onReSize', 'onLoad', 'onPageLoad', 'setTargets','initializeEvents','reloadParentEvents','keydown');


            this.on("completed:page", function (e) {
                console.log("data-enable-on-complete " + $('[data-enable-on-complete]').length);
		//@since 1.0
                $('[data-enable-on-complete]').removeClass('is-disabled').attr("aria-disabled",false);
            });

            this.initializePageComponents();
            this.initializeChild();


            window.onbeforeunload = function(event){self.exitcourse();};

            //Do not have a initialize function within your application.
            //Use Initialize child instead
        },


        /**
         * This function needs to be called for the screen to start initializing. THis allows all initializations to run before the init is called
         (window.bazingaApp.models.api.getPageScore());
         (window.bazingaApp.models.api.getPageCompletion());
         (window.bazingaApp.models.api.getPageData());
         (window.bazingaApp.models.api.getPageCount());
         (window.bazingaApp.models.api.getAllPages());
         (window.bazingaApp.models.api.getInfo('role'));
         (window.bazingaApp.models.api.getInfo('preferred'));
         (window.bazingaApp.models.api.getInfo('suspendData'));
         (window.bazingaApp.models.api.getInfo('currentScreen'));
         (window.bazingaApp.models.api.getInfo('previousScreen'));
         (window.bazingaApp.models.api.getPreviousScreen());
         (window.bazingaApp.models.api.getPageDataForPage());
         (window.bazingaApp.models.api.getConfigForScreen());
         (window.bazingaApp.models.api.getNavForScreen());
         (window.bazingaApp.models.api.getNavForScreen(id));
         (window.bazingaApp.models.api.getNavForScreenByRole());
         (window.bazingaApp.models.api.getNavForScreenByRole('default'));
         (window.bazingaApp.models.api.getNavForScreenByRole('default','home'));
         (window.bazingaApp.models.api.getDecision('next'));
         (window.bazingaApp.models.api.getDecision('back'));
         (window.bazingaApp.models.api.getDecision('complete'));
         (window.bazingaApp.models.api.getDecision('complete','default','home'));
         (window.bazingaApp.models.api.getNavForScreen('home'));
         (window.bazingaApp.models.api.getNavForScreenByRole());
         (window.bazingaApp.models.api.getNavForScreenByRole('default'));
         (window.bazingaApp.models.api.getNavForScreenByRole('default','1-1'));
         (window.bazingaApp.models.api.getDecision('complete','default','1-1'));
         (window.bazingaApp.models.api.shutdown());
         (window.bazingaApp.models.api.getRole());
         window.bazingaApp.models.api.setPageScore(22)
         window.bazingaApp.models.api.setPageCompletion('completed')
         window.bazingaApp.models.api.setPageData('xxxxx')
         window.bazingaApp.models.navigation.setCurrentPool(poolId)
         window.bazingaApp.models.navigation.getCurrentPool()
         * Load the application
         */
        load: function () {
            console.log("Loading application view");
            var self = this;
            self.backstretch = new window.bazingaApp.views.backstretchView();
            window.bazingaApp.preloader = self.preloader = new window.bazingaApp.views.preloaderView();


            window.bazingaApp.models.api = self.api = new window.bazingaApp.models.API();

            self.preloader.setLabel('Loading').setPercentage(0);

            $.getJSON("../config/config.json", function (courseConfig) {
                //course config is nwo set up
                window.bazingaCourseConfig = self.courseConfig = courseConfig;

                window.bazingaApp.models.api.once('initialize:dataModel', function () {
                    self.preloader.incrementTo(15);
                });
                window.bazingaApp.models.api.once('initialize:api', function () {
                    self.preloader.incrementTo(30);
                });
                window.bazingaApp.models.api.once('initialize:handshake', function () {
                    self.preloader.incrementTo(45);
                });
                window.bazingaApp.models.api.once('initialize:suspendData', function () {
                    self.preloader.incrementTo(60);
                });
                window.bazingaApp.models.api.once('initialize:page', function () {
                    self.preloader.incrementTo(75);
                });


                //The api loads the following
                //courseConfig
                //Nav file
                //Set up bread crumbs
                //Populates and renders menu
                //initializeAccessibility
                //Calls loaded

                window.bazingaApp.models.api
                    .load(courseConfig)
                    .always(self.onLoad);
            });
        },

        onLoad: function () {
            var self = this;

            self.pageConfig = (window.bazingaApp.models.api.getNavForScreen());

            if (self.pageConfig) {
                self.pageId = self.pageConfig.id;
                if (self.pageConfig.visibility) {
                    if (self.pageConfig.visibility.breadcrumbs) {
                        self.breadCrumbs = window.bazingaApp.models.api.getCrumbs();
                        self.$el.find('[data-bread-crumb]').html(self.getBreadCrumbs());
                    }
                    else {
                        self.$el.find('[data-bread-crumb]').hide();
                    }
                    self.nav = window.bazingaApp.models.api.getNav();
                    self.menuData = self.populateMenu(self.nav);
                    if (self.pageConfig.visibility && (!_.has(self.pageConfig.visibility, 'blockMenuRender') || self.pageConfig.visibility.blockMenuRender == false)) {
                        self.renderMenu();
                    }
                }

            }

            self.beforePageLoad();
            self.beforeAbstractLoad();

            self.preloader.incrementTo(100);
            self.initializeAccessibility();

            self.setTargets();
            self.onPageLoad();
            self.abstractLoads();
            self.loaded();
        },


        keydown : function(e){
            var self=this,
                $tgt,
                prevent = (function(){
                    if(e.ctrlKey || !self.pageConfig || !_.has(self.pageConfig,'blockers') || !_.has(self.pageConfig.blockers, 'keys')){
                        return false;
                    }
                    if(_.indexOf(self.pageConfig.blockers.keys,(''+ e.which)) < 0){return false;}
                    //Do nothing
                    return true;
                })();

            if(prevent){
                e.preventDefault();
                return false;
            }

            //Accessibility
            if (e.ctrlKey && ( e.which === 39)) {
                self.goNext();
            }
            if (e.ctrlKey && ( e.which === 37)) {
                self.goBack();
            }

            // accessibility
            $tgt = $(e.target);
            if($tgt.hasClass('js-skip-link')){
                if (e.which === 32 || e.which === 13) {
					//@since 1.0 fixed by Ali for pages with multiple skip targets
                    $(".js-skip-focus:visible:first").focus();
                }
            }


        },
        abstractLoads : function(){
            //Override this in abstract views
        },

        setTargets: function () {
            var suspendData = (window.bazingaApp.models.api.getSuspendData()),
                hashed = {
                    'completion': _('completion').getKeyHash(),
                    'completed': _('completed').getKeyHash(),
                    'passed': _('passed').getKeyHash(),
                    'incomplete': _('incomplete').getKeyHash()
                },
                lockables;

            if ($('[data-completion-target]').length !== 0) {
                $('[data-completion-target]').each(function () {

                    var $el = $(this),
                        data = $el.data('completionTarget'),
                        split,
                        completed,
                        isIncomplete = false;


                    if (!data) {
                        return true;
                    }
                    split = (""+data).split(",");
                    if (split.length == 0) {
                        return true;
                    }


                    _.each(split, function (tgt) {
                        var sus = _.where(suspendData, {id: tgt});
                        if (sus.length == 0) {
                            completed = false;
                            return false;
                        }
                        sus = _.first(sus);

                        if (!_.has(sus, hashed.completion) || !sus[hashed.completion] || ([hashed.completed , hashed.passed].indexOf(sus[hashed.completion]) == -1)) {
                            completed = false;
                            return false;
                        }

                        completed = true;
                        return true;
                    });

                    if (completed) {
                        $el.addClass('is-completed');
                    }
                    else {
                        $el.removeClass('is-completed');
                    }
                });

            }

            if ($('[data-locked-if]').length > 0) {
                lockables = (window.bazingaApp.models.api.getLockables());
                $('[data-locked-if]').each(function () {
                    var $el = $(this),
                        data = $el.data('lockedIf');

                    if (_.has(lockables, data) && lockables[data].locked == true) {
                        $el.addClass('is-disabled');
                    }
                    else {
                        $el.removeClass('is-disabled');
                    }
                });
            }


            if($('[data-available-pool]').length > 0){
                var currentPool =  ((window.bazingaApp.models.api.getCurrentPool())+"");

                $('[data-available-pool]').each(function () {
                    var $el = $(this),
                        data = $el.data('availablePool'),
                        dataSplit;


                    dataSplit= (data && data != '') ? (data+"").split(',') : [];

                    if(dataSplit.length == 0){return true;}

                    console.log("IX "+  _.indexOf(dataSplit,currentPool));
                    if(_.indexOf(dataSplit,"*") >= 0 || _.indexOf(dataSplit,currentPool) >= 0){
                        //$el.show();
                        $el.fadeIn();
                    }
                    else{
                        $el.hide();
                    }
                });
            }

        },
        beforePageLoad : function(){

        },
        beforeAbstractLoad : function(){

        },

        onPageLoad: function () {
            var gesture = $(".gesture"),
                count,
                mobileDevice = /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);

            if(gesture.length > 0){
                _.each(gesture,function(g){
                    var $g=$(g);

                    if ($g.text() === "Click" && mobileDevice == true) {
                        $g.text("Tap");
                    }
                    else if ($g.text() === "click" && mobileDevice == true) {
                        $g.text("tap");
                    }
                });
            }


            //If you override this function. make sure to call the super parent. Or you can just overrdie the loaded function
            var self = this,
                previousCompletion = 'na',
				pageCount = window.bazingaApp.models.api.getPageCount(),
                hashed = {
                    completed: _('completed').getKeyHash(),
                    passed: _('passed').getKeyHash()
                },
                preventAutoComplete = !self.pageConfig || !self.pageConfig.track || !_.has(self.pageConfig.track,'manual_completion') || self.pageConfig.track.manual_completion;


            if (self.pageConfig && self.pageConfig.type !== 'page') {
                return this;
            }

            if(_.has(self.pageConfig,'conditions') && _.has(self.pageConfig['conditions'],'countIf')){
				var handleCountFunctions = function(cnt){
					cnt = cnt || {};
					if(cnt.show){
						$(cnt.show).fadeIn();
					}
					if(cnt.hide){
						$(cnt.hide).hide();
					}

                    var score = 0,
                        status = "failed",
                        committable = [];



                    //Commit the score and status if the page has conditions
                    if(cnt.commitScore){
                        var value = cnt.commitScore.value || "primary",
                            multiplier = cnt.commitScore.multiplier || 1;

                        switch(value){
                            case 'primary':
                                score = window.bazingaApp.models.api.getPrimaryScore() || 0;
                                break;
                            case 'secondary':
                                score = window.bazingaApp.models.api.getSecondaryScore() || 0;
                                break;
                            case 'tertiary':
                                score = window.bazingaApp.models.api.getTertiaryScore() || 0;
                                break;
                            default:
                                score = _.isNumber(cnt.commitScore) ? cnt.commitScore  : 0;
                                break;
                        }

                        committable.push({key:'score',value:(score*multiplier)});
                    }

                    if(cnt.commitStatus){
                        var statusCondition   = cnt.commitStatus,
                            masteryScore = statusCondition.masteryScore || 100,
                            type = statusCondition.type || "passed-failed";

                        if(type == "passed-failed"){
                            if(score <= masteryScore){
                                status = "failed";
                            }
                            else{
                                status = "passed";
                            }
                        }
                        else{
                            if(score <= masteryScore){
                                status = "incomplete";
                            }
                            else{
                                status = "completed";
                            }
                        }

                        committable.push({key:'status',value:status});
                    }


                    if(committable.length !== 0){
                        window.bazingaApp.models.api.setMultiple(committable);
                    }
                }
                //handle count based variables
				_.each(self.pageConfig['conditions']['countIf'], function (cnt) {
					if(!cnt.condition || cnt.count=== undefined){return true;} 
					
					var condition = cnt.condition || "=";
					
					switch(condition){
						case "<":
							if(pageCount < cnt.count) {handleCountFunctions(cnt);}
							break;
						case ">":
							if(pageCount > cnt.count) {handleCountFunctions(cnt);}
							break;
						case "<=":
							if(pageCount <= cnt.count) {handleCountFunctions(cnt);}
							break;
						case ">=":
							if(pageCount >= cnt.count) {handleCountFunctions(cnt);}
							break;
						case "=":
						default:
							if(pageCount == cnt.count){handleCountFunctions(cnt);}
							break;
					}
				});

            }
            previousCompletion = window.bazingaApp.models.api.getPageCompletion();

            previousCompletion = previousCompletion || 'na';

            //disable any thing that needs to be disabled on enter
	    //@since 1.0
            $('[data-disable-on-enter]').addClass('is-disabled').attr("aria-disabled",true);
            //one time trigger
            if ([hashed.completed, hashed.passed].indexOf(previousCompletion) > -1) {
                //No calculations necessary
                //page is already completed
                self.trigger("completed:page", self, {target: self});
                return this;
            }

            if (self.interactions.length == 0  && !preventAutoComplete) {
                console.info("Page completed . \\o//");
                window.bazingaApp.models.api.setPageCompletion('completed');
                self.trigger("completed:page", self, {target: self});
                return this;
            }



        },
        /**
         * @private : Do not change
         * @param menu
         * @returns {*}
         */
        setMenu: function (menu) {
            this.menu = menu;
            return this;
        },
        /**
         * Initialize all page components on screen
         */
        initializePageComponents: function () {
            //popup view
            var self = this,
                completionCheck = function (tgt, e) {
                    var tgtType = ((typeof tgt.getComponentType == 'function') ? tgt.getComponentType() : null);

                    tgt.off(null,completionCheck);
                    //for who ever wants to listen
                    self.trigger('completed:component', self, {target: tgt, source: e, tgtType: tgtType});
                    self.completionCheck(self, {target: tgt, source: e, tgtType: tgtType });

                    //Send page completion
                    if (tgt.getIsCompleted()) {
                        self.checkPageCompletion();
                    }
                };

            //=======Audios===============
            self.audio = new window.bazingaApp.views.audioView();
            self.audio.load();

            if (self.audio.isEnabled()) {
                this.interactions.push(self.audio);
                self.audio.on('completed:audio', completionCheck);
            }

            //=======Videos===============
            self.video = new window.bazingaApp.views.videoView();
            self.video.load();

            if (self.video.isEnabled()) {
                this.interactions.push(self.video);
                self.video.on('completed:video', completionCheck);
                console.log(self.video.isEnabled() + " - video");
            }


            //=======Popups===============
            self.popup = new window.bazingaApp.views.popupView();

            if (self.popup.isEnabled()) {
                this.interactions.push(self.popup);
                self.popup.on('completed:reveal', completionCheck);
                self.popup.on('data:popup',function(context,options){
                    if(!options){return false};
                    self.initializeEvents(options.$tgt);
                });
            }


            //=======Checkboxes===============
            self.checkboxes = new window.bazingaApp.views.checkboxView();
            if (self.checkboxes.isEnabled()) {
                self.checkboxes.on('completed:checkboxes', completionCheck);
                this.interactions.push(self.checkboxes);
            }

            //=======FILLUPS===============
            self.fillups = new window.bazingaApp.views.fillupView();
            if (self.fillups.isEnabled()) {
                self.fillups.on('completed:fillups', completionCheck);
                this.interactions.push(self.fillups);
            }


            //=======??s===============
            self.tick = new window.bazingaApp.views.tickView();
            if (self.tick.isEnabled()) {
                self.tick.on('completed:tick', completionCheck);
                this.interactions.push(self.tick);
            }

            //=======Carousel===============
            self.carousel = new window.bazingaApp.views.carouselView();
            self.carousel.load();

            if (self.carousel.isEnabled()) {
                this.interactions.push(self.carousel);
                self.carousel.on('completed:carousel', completionCheck);
            }
            
            //=======Checkpoint===============
            self.checkpoint = new window.bazingaApp.views.checkPointView();

            if (self.checkpoint.isEnabled()) {
            	this.interactions.push(self.checkpoint);
                self.checkpoint.on('completed:checkpoint', completionCheck);
            }

            self.initializeChildComponents();


            //Hide any pool specific elements by default
            $('[data-available-pool]').hide();
        },

        /**
         * Checks to see if the page is completed.
         * Page is completed when all components in screen are completed
         * @returns {*}
         */
        checkPageCompletion: function () {

            //Checking page completion
            if (this.interactions.length == 0) {
                return true;
            }

            var self = this,
                completed = true,
                previousCompletion = window.bazingaApp.models.api.getPageCompletion(),
                hashed = {
                    completed: _('completed').getKeyHash(),
                    passed: _('passed').getKeyHash()
                };

            previousCompletion = previousCompletion || 'na';

            if ([hashed.completed, hashed.passed].indexOf(previousCompletion) > -1) {
                //No calculations necessary
                return this;
            }

            _.each(self.interactions, function (it) {
                if (typeof it.getIsCompleted !== 'function') {
                    //@todo
		    console.error("FIX YOUR CODE! Something broke. get is completed doesn't exist in the component");
                }

                if (!it.getIsCompleted()) {
                    completed = false;
                    return false;
                }
            });


            if (completed) {
                console.info("Page completed . \\o//");
                window.bazingaApp.models.api.setPageCompletion('completed');
                self.trigger("completed:page", self, {target: self});
            }
            else {
                window.bazingaApp.models.api.setPageCompletion('incomplete');
            }

            return this; 
        },


        /**
         * @private. Dont change
         */
        initializeAccessibility: function () {
            var self = this;
            //check the keydown function to add any keydown / class specific accessibility stuff
        },

        /**
         * Set up resize of the menu
         */
        initializeMenu: function () {
            // Menu Dropdown - done using screen height
            var self = this;
            $(window).on("resize", this.onReSize);
            self.onReSize();
        },


        populateMenu: function (nav) {
            if (!this.menu || this.menu.$el.length == 0) {
                return false;
            }
            var self = this,
                menu = [],
                allPages = window.bazingaApp.models.api.getAllPages(),
                currentPool = window.bazingaApp.models.api.getCurrentPool();


            _.each(nav, function (v, k) {
                var m = (_.first(_.where(allPages, {id: k})));
                if (m && m.visibility && m.visibility.menu) {
                    if(!currentPool){
                        menu.push(m);
                    }
                    else{
                        if(!m.track || !m.track.pool || ((m.track.pool+"")===(currentPool+""))){
                            menu.push(m);
                        }
                    }
                }
            });


            return menu;
        },

        goNext: function () {
            var next = (window.bazingaApp.models.api.getDecision('next'));
            nextData = (window.bazingaApp.models.api.getNavForScreen(next || window.bazingaCourseConfig.defaultHome || 'home'));

            if (!nextData) {
                console.error("Next location not located");
            }

            window.location.replace(nextData.resource);
        },
        goBack: function () {
            var back = (window.bazingaApp.models.api.getDecision('back'));
            backData = (window.bazingaApp.models.api.getNavForScreen(back || window.bazingaCourseConfig.defaultHome || 'home'));

            if (!backData) {
                console.error("back location not located");
            }

            window.location.replace(backData.resource);
        },

        exitcourse: function () {
            (window.bazingaApp.models.api.shutDown())
            return this;
        },

        goto: function (decision) {
            var self = this,
                decisionData;

            if (!decision) {
                return false;
            }

            decisionData = (window.bazingaApp.models.api.getNavForScreen(decision || window.bazingaCourseConfig.defaultHome || 'home'));


            if (!decisionData || !decisionData.resource) {
                decisionData = (window.bazingaApp.models.api.getNavForDialog(decision));
                if (!decisionData || !decisionData.resource) {
                    return false;
                }
            }

            window.location.replace(decisionData.resource);
            return self;
        },
        triggerData: function (e) {
            var self = this,
                $target = $(e.currentTarget),
                decision = $target.data('trigger');

            if (!decision) {
                return false;
            }


            if (typeof self[decision] == 'function') {
                self[decision].apply(self, $target.data());
            }

        },
        gotoDecisionById: function (e) {
            var $target = $(e.currentTarget),
                decision = $target.data('goto'),
                decisionData;


            if ($target.hasClass('is-disabled')) {
                return false;
            }


            if (!decision) {
                return false;
            }
            return this.goto(decision);
        },
        gotoDecisionPoint: function (e) {
            var $target = $(e.currentTarget),
                decision = $target.data('decision'),
                decision = (window.bazingaApp.models.api.getDecision(decision)),
                decisionData;

            if ($target.hasClass('is-disabled')) {
                return false;
            }
            return this.goto(decision);
        },


        //=====================================================================================
        //=========================END PRIVATE FUNCTIONS====================
        //=====================================================================================

        /**
         * Anything below here can be overrided in child pages
         *
         */
        onReSize: function () {
            var windowHeight = $(window).height(),
                menuHeight = windowHeight - 115;

            // console.log("ON RESIZE height being reset on menu");
            //$(".navbar-navigation").css("height", menuHeight);
        },

        loaded: function () {
            //override this on your page view
            console.log("loaded");
        },
        render: function () {

        },
        completionCheck: function (context, options) {
            //Override this for checking which component gets completed
        },
        initializeChild: function () {
            //override this
        },
        initializeChildComponents: function () {
            //override this
        },
        /**
         * This will return AEC type bread crumbs (Topic Name Page 1 of 6)
         * @returns {string}
         */
        getBreadCrumbs: function () {
            //Override this and use this
            var self = this,
                crumbs = self.breadCrumbs,
                first,
                listItems = null,
                crumbString = '',
                ix;


            if (!crumbs) {
                return "";
            }


            if (crumbs.length == 0) {
                return "<strong>" + self.pageConfig.title + "</strong>";
            }

            first = (_.first(crumbs));

            if (!first) {
                return "";
            }

            listItems = (_.filter(first.items, function (it) {
                return it.visibility && it.visibility.breadcrumbs;
            }));


            ix = (_.findIndex(listItems, function (it) {
                return "" + it.id == "" + self.pageId;
            }));

            var txt = first.title ? (first.title + " -") : "";

            if (ix == -1) {
                return  "<strong>" + txt + "</strong>";
            }

            crumbString = "<strong>" + txt + "</strong> PAGE " + (ix + 1) + " of " + listItems.length;
            return crumbString;
        },
        reattempt: function (e) {
            var target = e.currentTarget,
                $target = $(target),
                self = this,
                promise;

            if ($target.data('reattempt') == 'current') {
                promise = window.bazingaApp.models.api.reattemptScreen();
                if(promise){
                    promise.always(function () {
                            if ($target.data('reattemptThenGoto')) {
                                self.goto($target.data('reattemptThenGoto'));
                            }
                        });
                }
            }
        },
        /**
         * render the menu / you can override this
         * @returns {*}
         */
        renderMenu: function () {
            var self = this,
                menu = self.menuData,
                allHtml = "",
                listIt,
                hashed = {
                    "completion": _('completion').getKeyHash(),
                    "completed": _('completed').getKeyHash(),
                    "passed": _('passed').getKeyHash()
                },
		//@since 1.0
                courseConfig = window.bazingaApp.models.api.getCourseConfig(),
                accessibilityVersion = (_.has(courseConfig,'accessibilityVersion') ? courseConfig['accessibilityVersion'] : 1),
                suspendData = window.bazingaApp.models.api.getSuspendData(),
                lockables = (window.bazingaApp.models.api.getLockables()),
                renderSubMenu = function (items) {
                    var html = "",
                        listItems = (_.filter(items, function (it) {
                            return it && it.visibility && it.visibility.menu;
                        }));

                    if (listItems.length == 0) {
                        return false;
                    }

                    _.each(listItems, function (v, k) {
                        var hasCompletion = (v.track && v.track.completion),
                            pageSuspendData = hasCompletion ? _.first(_.where(suspendData, {id: v.id})) : false,
                            isCompleted = true;

                        if (!pageSuspendData || !pageSuspendData[hashed.completion] || (![hashed.completed, hashed.passed].indexOf(pageSuspendData[hashed.completion]) == -1)) {
                            //not attempted or incomplete
                            isCompleted = false;
                        }

                         html += '<li>' +
                            '<a href="#" data-sub-menu="1" class="dropdown-button ' + (isCompleted ? 'is-completed' : '') + '"' +
                            ' tabindex="' + self.menu.tabIndex + '" data-goto="' + v.id + '"> <span class="icon-tick"><span class="a11y-assist-text">- completed</span></span>' + v.title + '</a>' +
                            '</li>';

                    });


                    return html;
                };


            self.menu.$el.empty();
		//@since 1.0
            if(accessibilityVersion >= 2){
                self.menu.tabIndex= "0";
            }

            listIt = (_.filter(menu, function (it) {
                return it && it.visibility && it.visibility.menu;
            }));


            _.each(listIt, function (v, k) {

                var hasCompletion = (v.track && v.track.completion),
                    pageSuspendData = hasCompletion ? _.first(_.where(suspendData, {id: v.id})) : false,
                    isCompleted = true,
                    subMenu;

                if (!pageSuspendData || !_.has(pageSuspendData, hashed.completion) || ([hashed.completed, hashed.passed].indexOf(pageSuspendData[hashed.completion]) < 0)) {
                    isCompleted = false;
                }


                var filtered = _.filter(v.items, function (it) {
                        return it && it.visibility && it.visibility.menu;
                    }),
                    isDisabled = (function () {
                        if (_.has(lockables, v.id) && lockables[v.id].locked == true) {
                            return true;
                        }
                        return false;
                    }());

                if (filtered.length > 1) {
                    allHtml += "<li>";

                    if (isDisabled) {
                        //render as a button
                        allHtml += '<a href="#"  class="dropdown-button menu--level1 is-disabled' + (isCompleted ? 'is-completed' : '') + '"' +
                            ' tabindex="' + self.menu.tabIndex + '">' + v.title + '<span class="icon-tick"><span class="a11y-assist-text">Is completed</span></span>' + '</a>';
                            
                    }
                    else {
                        subMenu = renderSubMenu(v.items);
                        if (subMenu) {
                            allHtml += '<a role="button" data-toggle="collapse" data-parent="#js-dropdown-' + v.id + '"' +
                                'href="#js-dropdown-' + v.id + '" aria-expanded="true" aria-controls="js-dropdown-' + v.id + '" ' +
                                'class="dropdown-button dropdown-button--topic menu--level1 ' + (isCompleted ? 'is-completed' : '') + '" tabindex="' + self.menu.tabIndex + '"><span class="icon-tick"></span>' + v.title + '</a>';


                            if (v.items && _.isArray(v.items) && v.items.length > 0) {
                                allHtml += '<ul id="js-dropdown-' + v.id + '" class="dropdown-level-2 collapse" >';
                                allHtml += subMenu;
                                allHtml += '</ul>';
                            } 

                        }
                    }

                    allHtml += "</li>"

                }
                else if (filtered.length == 1) {
                    var first = _.first(v.items);

                    if (isDisabled) {
                        //render as a button
                        allHtml += '<li>' +
                            '<a href="#"  aria-disabled="true" class="dropdown-button menu--level1 is-disabled' + (isCompleted ? 'is-completed' : '') + '"' +
                            ' tabindex="' + self.menu.tabIndex + '"> <span class="icon-tick"><span class="a11y-assist-text">- completed</span></span>' + v.title + '</a>' +
                            '</li>';
                    }
                    else {
                        //render as a button
                        allHtml += '<li>' +
                            '<a href="#"  data-sub-menu="1" class="dropdown-button menu--level1 ' + (isCompleted ? 'is-completed' : '') + '"' +
                            ' tabindex="' + self.menu.tabIndex + '"data-goto="' + first.id + '"> <span class="icon-tick"><span class="a11y-assist-text">- completed</span></span>' + v.title + '</a>' +
                            '</li>';
                    }
                }


            });

            self.menu.$el.html(allHtml);

            if (self.menu.$el.find('[data-sub-menu]').length) {
                self.menu.$el.find('[data-sub-menu]').on('click touchend', function (e) {
                    self.gotoDecisionById(e);
                });
            }

            self.initializeMenu(); 
            return self;
        }
    });

}());

(function(){
    window.bazingaApp.views.dialogContinueView =window.bazingaApp.views.appView.extend({
        el:$('#js-app'),
        abstractLoads : function(){
            $('[tabindex="1"]:first').focus();
        },
        loaded: function(){
            //override this on your page view

        }
    });

}());
(function(){
    window.bazingaApp.views.dialogResourcesView =window.bazingaApp.views.appView.extend({
        el:$('#js-app'),
        abstractLoads : function(){
            $('[tabindex="1"]:first').focus();
        },
        loaded: function(){
            //override this on your page view

        }
    });

}());
(function(){
    window.bazingaApp.views.dialogResetView =window.bazingaApp.views.appView.extend({
        el:$('#js-app'),
        abstractLoads : function(){
            $('[tabindex="1"]:first').focus();
        },
        loaded: function(){
            //override this on your page view

        }
    });

}());
(function(){
    window.bazingaApp.views.dialogCompleteView =window.bazingaApp.views.appView.extend({
        el:$('#js-app'),
        abstractLoads : function(){
            $('[tabindex="1"]:first').focus();
        },
        loaded: function(){
            //override this on your page view

        }
    });

}());
(function(){
    window.bazingaApp.views.dialogExitView =window.bazingaApp.views.appView.extend({
        el:$('#js-app'),
        abstractLoads : function(){
            $('[tabindex="1"]:first').focus();
        },
        loaded: function(){
            //override this on your page view

        }
    });

}());
(function(){
    window.bazingaApp.views.dialogHelpView =window.bazingaApp.views.appView.extend({
        el:$('#js-app'),
        abstractLoads : function(){
		
            $('[tabindex="1"]:first').focus();
        },
        loaded: function(){
            //override this on your page view

        }
    });

}());
(function(){
    window.bazingaApp.views.abstractSplash =window.bazingaApp.views.appView.extend({
        el:$('#js-app'),
        abstractLoads : function(){
            $('[tabindex]:first').focus();
        }
    });

}());
(function(){
    window.bazingaApp.views.abstractClickAudioView =window.bazingaApp.views.appView.extend({
        el:$('#js-app'),
        abstractLoads : function(){
            //console.log(window.bazingaApp.models.api.getCourseConfig());
            var footer = $(".footbar"),
                tabIndexNum = "850",
                playButton = footer.find(".mejs-playpause-button").children().first(),
                volumeButton = footer.find(".mejs-volume-button").children().first(),
                itemsArr = [playButton, volumeButton],
                self=this,
                i,
                courseConfig = window.bazingaApp.models.api.getCourseConfig(),
                accessibilityVersion = _.has(courseConfig,'accessibilityVersion') ? courseConfig['accessibilityVersion'] : 1;

            if(accessibilityVersion  >= 2){
                //do stuff for custom module accessibility
                tabIndexNum =  '0';
            }
            //start page level js here


            $('[data-audio-play]').click(function(elem) {
                var $elem = $(elem), data = $elem.data();

                if($elem.hasClass('is-disabled')){return false;}

                $elem.addClass('is-disabled');

                if(_.has(data,'src') && self.audio){
                    self.audio.resetSrc(data.src);
                }
                else{
                    //self.audio.play();
                    playButton.click().focus();
                    $('[data-audio-play]').unbind("click");

                }
            });

            // Fixing tabindex for player
            for (i=0; i<itemsArr.length; i++) {
                itemsArr[i].attr("tabindex", tabIndexNum);
            }

            // skip to content /local
            //$(".js-skip-link").click(function() { $(".js-skip-focus").focus() });
        }
    });

}());